/*****************************************
 * Steven Stanek
 * July 7, 2001
 * YZBufferedDraw.c
 * Version 0.0.0a
 *****************************************/
 
#include "YZBufferedDraw.h"
#include <assert.h>

struct yzBufferCDT
{	
	void *dBuffer;
	int width, height, depth;
};


//For these helpers, if transValue = null if no transparency
static void addToGraphicsBuf16Bit(yzGraphicsBuffer aBuf, int x, int y, unsigned short *data, int width, int height,
					         int rowbytes, unsigned short *transValue);
static void addToGraphicsBuf32Bit(yzGraphicsBuffer aBuf, int x, int y, unsigned long *data, int width, int height,
							 int rowbytes, unsigned long *transValue);

yzGraphicsBuffer createNewBuffer(int width, int height, int depth)
{
	yzGraphicsBuffer aBuf;
	
	assert(aBuf = malloc(sizeof(struct yzBufferCDT)));
	assert(aBuf->dBuffer = malloc(width * height * depth));
	aBuf->width = width;
	aBuf->height = height;
	aBuf->depth = depth;
	
	assert(sizeof(unsigned long) == 4);
	assert(sizeof(unsigned short) == 2);
	assert(depth == 4 || depth == 2);
	
	return aBuf;
}


void addToGraphicsBuffer(yzGraphicsBuffer aBuf, int x, int y, void *data, int width, int height, int rowbytes)
{
	if(aBuf->depth == 2)
		addToGraphicsBuf16Bit(aBuf, x,  y, (unsigned short *) data, width, height, rowbytes, 0);
	else if(aBuf->depth == 4)
		addToGraphicsBuf32Bit(aBuf, x, y, (unsigned long *) data, width, height, rowbytes, 0);
}


void addToGraphicsBufferWithTransparency(yzGraphicsBuffer aBuf, int x, int y, void *data, int width, 
	int height, int rowbytes, void *transValue)
{
	if(aBuf->depth == 2)
		addToGraphicsBuf16Bit(aBuf, x, y, (unsigned short *) data, width, height, rowbytes, (unsigned short *) transValue);
	else if(aBuf->depth == 4)
		addToGraphicsBuf32Bit(aBuf, x, y, (unsigned long *) data, width, height, rowbytes, (unsigned long *) transValue);
}


void drawGraphicsBuffer(yzGraphicsBuffer aBuf, int x, int y)
{
	yzDisplayBits(x, y, aBuf->width, aBuf->height, aBuf->depth * aBuf->width, aBuf->dBuffer);
}


void freeGraphicsBuffer(yzGraphicsBuffer toFree)
{
	free(toFree->dBuffer);
	free(toFree);
}


//Static helpers
static void addToGraphicsBuf16Bit(yzGraphicsBuffer aBuf, int x, int y, unsigned short *data, int width, int height,
					         int rowbytes, unsigned short *transValue)
{
	int i, j;
	unsigned short *actualBuffer;
	
	actualBuffer = aBuf->dBuffer;
	
	for(i=0; i  < height && i + y < aBuf->height; i++)
	{
		for(j=0; j < rowbytes && j + x < aBuf->width; j++)
		{
			if(!transValue ||  data[i * width + j] != *transValue)
				actualBuffer[((i + y) * aBuf->width) + (j + x)] = data[i * width + j];
		}
	}
}


static void addToGraphicsBuf32Bit(yzGraphicsBuffer aBuf, int x, int y, unsigned long *data, int width, int height,
							 int rowbytes, unsigned long *transValue)
{
	int i, j;
	unsigned long *actualBuffer;
	
	actualBuffer = aBuf->dBuffer;
	
	for(i=0; i  < height && i + y < aBuf->height; i++)
	{
		for(j=0; j < rowbytes && j + x < aBuf->width; j++)
		{
			if(!transValue ||  data[i * width + j] != *transValue)
				actualBuffer[((i + y) * aBuf->width) + (j + x)] = data[i * width + j];
		}
	}
}