/********************************************** 
 * Steven Stanek
 * July 5, 2001
 * YZRescuer
 * Version: 0.0.0c
 **********************************************/

#include	<stdlib.h>
#include	<time.h>
#include	<yz/window.h>
#include	<yz/draw.h>
#include 	<yz/cursor.h>
#include 	<math.h>
#include	"YZRescuerFileReader.h"
#include	"YZBufferedDraw.h"

#if macintosh
	#undef feof
#endif

#ifndef bool
	#define bool int
	#define true 1
	#define false 0
#endif

//Window Related Constants
#define	WIN_WIDTH	480
#define WIN_HEIGHT	320
#define WIN_NAME	"YZRescuer"

//Game Constants
#define REFRESHES_IN_SEC	40
#define REFRESH_TIME		CLOCKS_PER_SEC / REFRESHES_IN_SEC
#define TOP_SPEED_COPTER	500  //Pixels Per Second
#define COPTER_V_IN_CYCLE	TOP_SPEED_COPTER / REFRESHES_IN_SEC
#define START_SHARKS		10
#define START_BOATS			15
#define SHARK_SPEED_IN_SEC	150
#define SHARK_V_IN_CYCLE	SHARK_SPEED_IN_SEC / REFRESHES_IN_SEC
#define SHARK_HIT_DISPLACE	40
#define SHARK_HIT_OFFSET	10
#define BOAT_SPEED_IN_SEC	80
#define BOAT_V_IN_CYCLE		BOAT_SPEED_IN_SEC / REFRESHES_IN_SEC
#define POINTS_PER_BOAT		50
#define POINTS_PER_SHARK	-75	
#define PAR_TIME_IN_SEC		10
#define SEC_SCORE			20
#define TRANS_COLOR_16		(19<<10) | (31<<5) | (31)
#define TRANS_COLOR_32		(156<<16) | (255<<8) | (255)

//All those structs
typedef struct
{
	yzImage aImage;
	int px, py; //Pres x & Pres y
	int lx, ly; //Last x & Last y
	int dx, dy; //Destination (for AI stuff only)
} rescuer_thing_t;

typedef struct
{
	file_data_t *dImages;
	yzGraphicsBuffer aBuf;
	rescuer_thing_t *chopper, *sharks, *boats; //1 Chopper but sharks & boats are arrays
	int num_boats, num_sharks;
	int score;
} game_data_t;


//Function Prototypes
window_t *initWindow(file_data_t *dFDAT);
game_data_t *initGameData();
void freeGameData(game_data_t *dGame);
void handleCursor(game_data_t *dGame);

//Drawing Functions
void drawAll(game_data_t *dGame);
void drawBackground(game_data_t *dGame);
void drawThing(rescuer_thing_t *thing, yzGraphicsBuffer aBuf, void *transColor);

//Handling Functions
void handleSharks(game_data_t *dGame);
void handleBoats(game_data_t *dGame);

//Other Functions
void weightedMove(rescuer_thing_t *dThing, int dx, int dy, int speed);
inline bool isCollision(rescuer_thing_t *thingA, rescuer_thing_t *thingB);
inline bool is1DOverlap(int a1, int a2, int b1, int b2);

//Code Begins
main()
{
	window_t *mainWindow;
	game_data_t *dGame;
	
	srand(clock());
		
	dGame = initGameData();
	mainWindow = initWindow(dGame->dImages);
	dGame->aBuf =  createNewBuffer(WIN_WIDTH, WIN_HEIGHT, yzGetDepth()/8);
	
	handleCursor(dGame);
	
	freeGameData(dGame);
	
	return 0;
}


window_t *initWindow(file_data_t *dFDAT)
{
	window_t *mainWindow;
	
	mainWindow = yzNewWindow(WIN_NAME, WIN_WIDTH, WIN_HEIGHT);
	yzSelectWindow(mainWindow);
	
	fastDrawAll(dFDAT);
	
	//drawBackground(dFDAT);
	
	return mainWindow;
}


//Note: Doesn't init aBut in game_data_t because window has not yet been created
//modify if work around is found
game_data_t *initGameData()
{
	game_data_t *dGame;
	int i;
	
	assert(dGame = malloc(sizeof(game_data_t)));
	assert(dGame->dImages = readRescuerFile(DATA_FN));
	dGame->score = 0;
	
	
	//Create Chopper
	assert(dGame->chopper = malloc(sizeof(rescuer_thing_t)));
	dGame->chopper->aImage = dGame->dImages->chopperImage;
	dGame->chopper->px = 0;
	dGame->chopper->py = 0;
	
	//Create Sharks & set num_sharks
	assert(dGame->sharks = malloc(sizeof(rescuer_thing_t) * START_SHARKS));
	dGame->num_sharks = START_SHARKS;
	
	for(i=0; i < START_SHARKS; i++)
	{
		dGame->sharks[i].aImage = dGame->dImages->sharkImage;
		dGame->sharks[i].px = rand() % WIN_WIDTH;
		dGame->sharks[i].py = rand() % WIN_HEIGHT;
		dGame->sharks[i].dx = rand() % WIN_WIDTH;
		dGame->sharks[i].dy = rand() % WIN_HEIGHT;
	}
	
	
	//Create Boats & set num_boats
	assert(dGame->boats = malloc(sizeof(rescuer_thing_t) * START_BOATS));
	dGame->num_boats = START_BOATS;
	
	for(i=0; i < START_BOATS; i++)
	{
		dGame->boats[i].aImage = dGame->dImages->boatImage;
		dGame->boats[i].px = rand() % WIN_WIDTH;
		dGame->boats[i].py = rand() % WIN_HEIGHT;
		dGame->boats[i].dx = rand() % WIN_WIDTH;
		dGame->boats[i].dy = rand() % WIN_HEIGHT;
	}
		
	return dGame;
}


void freeGameData(game_data_t *dGame)
{
	emptyFileData(dGame->dImages);
	freeGraphicsBuffer(dGame->aBuf);
	free(dGame->chopper);
	free(dGame->sharks);
	free(dGame->boats);
	free(dGame);
}


void handleCursor(game_data_t *dGame)
{
	int lclock, startClock, x, y;
	char outarr[128];
	int i=0, numsecs; //Error Checking / Delete Later
	
	
	
	startClock = lclock = clock();
	
	while(1)
	{
		yzQueryPointer(&x, &y);
		
		if(clock() - lclock > REFRESH_TIME && x != -1)
		{
			weightedMove(dGame->chopper, x, y, COPTER_V_IN_CYCLE);
			
			handleSharks(dGame);
			handleBoats(dGame);			
			drawAll(dGame);
			
			i++; //Error Check / Delete Me
			
			if(dGame->num_boats == 0)
				break;
			
			lclock = clock();
		}
	}
	
	
	
	numsecs = (clock() - startClock) / CLOCKS_PER_SEC; //Error Checking / Delete me
	
	
	dGame->score -= SEC_SCORE * (clock() - startClock) / CLOCKS_PER_SEC;
	dGame->score += PAR_TIME_IN_SEC * SEC_SCORE;
	
	sprintf(outarr, "Your final score is %d, average frameRate = %d", dGame->score, i/numsecs);
	
	yzText(WIN_WIDTH /4, WIN_HEIGHT/2, outarr);
	
	while(yzQueryPointer(&x, &y) != 3)
	{
	}
}


void drawAll(game_data_t *dGame)
{
	int i;
	unsigned long trans32;
	unsigned short trans16;
	void *toPass;
	
	trans32 = TRANS_COLOR_32;
	trans16 = TRANS_COLOR_16;
	
	toPass = (yzGetDepth() == 16) ? (void*) &(trans16) : (void*) &(trans32);
	
	drawBackground(dGame);
	
	drawThing(dGame->chopper, dGame->aBuf, toPass);
	
	//Draw Sharks
	for(i=0; i < dGame->num_sharks; i++)
		drawThing(&(dGame->sharks[i]), dGame->aBuf, toPass);
	
	//Draw Boats
	for(i=0; i < dGame->num_boats; i++)
		drawThing(&(dGame->boats[i]), dGame->aBuf, toPass);
		
	drawGraphicsBuffer(dGame->aBuf, 0, 0);
}


void drawBackground(game_data_t *dGame)
{
	int i, j, width, height;
	void *image;
	
	width = getImageWidth(dGame->dImages->backgroundImage);
	height = getImageHeight(dGame->dImages->backgroundImage);
	image = getFastDrawArray(dGame->dImages->backgroundImage);
	
	for(i=0; i < WIN_HEIGHT; i+=height)
		for(j=0; j < WIN_WIDTH; j+=width)
			addToGraphicsBuffer(dGame->aBuf, j, i, image, width, height, width);
}


void drawThing(rescuer_thing_t *thing, yzGraphicsBuffer aBuf, void *transColor)
{
	addToGraphicsBufferWithTransparency(aBuf, thing->px, thing->py, getFastDrawArray(thing->aImage), 
		getImageWidth(thing->aImage), getImageHeight(thing->aImage), getImageWidth(thing->aImage), transColor);
	
	thing->lx = thing->px;
	thing->ly = thing->py;
}


void handleSharks(game_data_t *dGame)
{
	int i;
	
	for(i=0; i < dGame->num_sharks; i++)
	{
		if(dGame->sharks[i].dx == dGame->sharks[i].px || dGame->sharks[i].dy == dGame->sharks[i].py)
		{
			dGame->sharks[i].dx = rand() % WIN_WIDTH;
			dGame->sharks[i].dy = rand() % WIN_HEIGHT;
		}
			
		weightedMove(&(dGame->sharks[i]), dGame->sharks[i].dx, dGame->sharks[i].dy, SHARK_V_IN_CYCLE);
		
		if(isCollision(&(dGame->sharks[i]), dGame->chopper))
		{
			dGame->sharks[i].px += (rand() %2 == 0 ? -1 : 1) * (SHARK_HIT_OFFSET + rand() % SHARK_HIT_DISPLACE);
			dGame->sharks[i].py += (rand() %2 == 0 ? -1 : 1) * (SHARK_HIT_OFFSET + rand() % SHARK_HIT_DISPLACE);
			
			dGame->score += POINTS_PER_SHARK;
		}
	}
}


void handleBoats(game_data_t *dGame)
{
	int i;
	
	for(i=0; i < dGame->num_boats; i++)
	{
		if(dGame->boats[i].dx == dGame->boats[i].px || dGame->boats[i].dy == dGame->boats[i].py)
		{
			dGame->boats[i].dx = rand() % WIN_WIDTH;
			dGame->boats[i].dy = rand() % WIN_HEIGHT;
		}
			
		weightedMove(&(dGame->boats[i]), dGame->boats[i].dx, dGame->boats[i].dy, BOAT_V_IN_CYCLE);
		
		if(isCollision(&(dGame->boats[i]), dGame->chopper) && dGame->num_boats != 0)
		{
			dGame->boats[i] = dGame->boats[dGame->num_boats-1];
			dGame->num_boats --;
			dGame->score += POINTS_PER_BOAT;
		}
		
	}
}


//Moves a thing toward dx dy with weighting.
//Speed is the max distance it can move in one refresh (this one).
void weightedMove(rescuer_thing_t *dThing, int dx, int dy, int speed)
{
	int difx, dify;
	double cydif, cxdif; //Calculated changes in x & y
	double difxtoy;
	
	difx = dx - dThing->px;
	dify = dy - dThing->py;
	
	if(dx <0 || dy < 0)
		return;
	
	//Simple Case: total dist is less than speed 
	else if((difx * difx + dify * dify) < speed * speed)
	{
		dThing->px = dx;
		dThing->py = dy;
	}
	
	//Complex Case
	else
	{
		if(difx==0)
		{
			cydif = speed;
			cxdif = 0;
		}
		else if(dify==0)
		{
			cydif = 0;
			cxdif = speed;
		}
		else
		{
			difxtoy = abs(difx / dify);
			cydif = sqrt(speed * speed / (difxtoy * difxtoy +1));
			cxdif = difxtoy * cydif;
		}
		
		//make sure signs are correct
		cydif *= dify / abs(dify);
		cxdif *= difx / abs(difx);
		
		dThing->px += cxdif;
		dThing->py += cydif;
	}
}


//Uses box method only, if overlap in both dirs returns true, else false
inline bool isCollision(rescuer_thing_t *thingA, rescuer_thing_t *thingB)
{
	return(is1DOverlap(thingA->px, thingA->px + getImageWidth(thingA->aImage), 
	   thingB->px, thingB->px + getImageWidth(thingB->aImage)) &&
	  is1DOverlap(thingA->py, thingA->py + getImageHeight(thingA->aImage),
	   thingB->py, thingB->py + getImageHeight(thingB->aImage)));
	
}


//Determines if there is overlaps between coords in a specific dimension
//a2 must be bigger than a1 and b2 must be bigger than b1
inline bool is1DOverlap(int a1, int a2, int b1, int b2)
{	
	return ((a1 > b1 && a1 < b2) || (a2 > b1 && a2 < b2) || (b1 > a1 && b1 < a2) || (b2 > a1 && b2 < a2));
}