/* YZiForm Editor
 * By: Steven Stanek
 * January 28, 2001
 * Version 0.0.1a
 */

#include	<time.h>
#include	<math.h>
#include 	"yziform.h"
#include 	"yziformutil.h"

#define		bool			int
#define		true			1
#define		false			0
#define 	PIXEL_SIZE 		2
#define 	MAX_WIDTH 	 	640/PIXEL_SIZE
#define		MAX_HEIGHT 		480/PIXEL_SIZE
#define		WINDOW_NAME 	"Image Editor Window"
#define		REDRAW_TIME		CLOCKS_PER_SEC / 20

//CW MEANS COLORS WINDOW
#define		CW_WIDTH		120
#define		CW_HEIGHT		120
#define		CW_NAME			"Colors"
#define		CW_SIDE_BUFFER	10
#define		CW_V_BUFFER		5
#define		CW_COLOR_W		CW_WIDTH - 2 * CW_SIDE_BUFFER
#define		CW_COLOR_H		20
#define		CW_FINAL_H		35
#define		CW_FINAL_SIDE	20
#define		CW_FINAL_W		CW_WIDTH - 2 * CW_FINAL_SIDE
#define		CW_EXIT_SIDE	15
#define		CW_TRANS_BUF	3 //Distance between edges of black box and T for transparency
#define		RECURSE_LIM		2000

//TW MEANS TOOLS WINDOW
#define		TW_WIDTH		120
#define		TW_NUM_H_BOXES	3
#define		TW_NUM_V_BOXES	3
#define		TW_HEIGHT		(TW_WIDTH / TW_NUM_H_BOXES)*TW_NUM_V_BOXES
#define		TW_ROW2_TOP		(TW_WIDTH / TW_NUM_H_BOXES)
#define		TW_ROW3_TOP		TW_HEIGHT / 3 * 2
#define		TW_NAME			"Tools"
#define 	TW_BOX_H_SPACER	TW_WIDTH/TW_NUM_H_BOXES
#define		TW_TOOL_BUFFER	5
#define		TW_TOOL_SIDE	(TW_WIDTH / TW_NUM_H_BOXES) - (2 * TW_TOOL_BUFFER)
#define		TW_TOOL_START	TW_TOOL_BUFFER
#define		TW_TOOL_END		TW_TOOL_SIDE + TW_TOOL_BUFFER
#define		TW_ROW2_START	TW_ROW2_TOP + TW_TOOL_START
#define		TW_ROW2_END		TW_ROW2_TOP + TW_TOOL_END
#define		TW_ROW3_START	TW_ROW3_TOP + TW_TOOL_START
#define		TW_ROW3_END		TW_ROW3_TOP + TW_TOOL_END
#define		TW_DOT_SIDE		PIXEL_SIZE
#define		TW_DOT_START	TW_HEIGHT/(2*TW_NUM_V_BOXES) - TW_DOT_SIDE
#define		TW_DOT_END		TW_HEIGHT/(2*TW_NUM_V_BOXES) + TW_DOT_SIDE
#define		TW_RECT_H_START	TW_BOX_H_SPACER + TW_TOOL_START
#define		TW_RECT_H_END	TW_BOX_H_SPACER + TW_TOOL_END
#define		TW_LINE_H_START TW_BOX_H_SPACER * 2 + TW_TOOL_START
#define		TW_LINE_H_END	TW_BOX_H_SPACER * 2 + TW_TOOL_END
#define		TW_SELECT_BUF	3
#define		BUF_DIF			2
#define		TW_SELECT_SIDE	TW_BOX_H_SPACER - (TW_SELECT_BUF *2)
#define		TW_SELECT_BTM	TW_ROW2_TOP - TW_SELECT_BUF
#define		TW_CIRCLE_RAD	12
#define		TW_CIRCLE_CY	(TW_ROW2_START + TW_ROW2_END)/2
#define		TW_CIRCLE_CX	(TW_LINE_H_START+TW_LINE_H_END)/2
#define		TW_FILENAME		"editortools.dat"
#define 	TW_FCIRC_CY		(TW_ROW3_START + TW_ROW3_END)/2
#define		TW_FCIRC_CX		(TW_DOT_START + TW_DOT_END)/2
#define		TW_ELPSE_WID	15
#define		TW_ELPSE_HGT	12
#define		TW_ELPSE_CX		(TW_RECT_H_START + TW_RECT_H_END)/2

//Save Stuff
#define		MAX_FN_LEN		64
#define		DEF_LN_LEN		7+2+5+1+5+1+5+2


typedef struct
{
	int width, height;
	color_t **image_data;
} ImageData_t;


typedef enum
{
	dot, rect, line, dropper, bucket, brush, circle, fillcirc, ellipse, fellipse
} Tool_t;

typedef void (*pixelFunction)(int x, int y, color_t *presColor, ImageData_t *image);

//Starting Functions
ImageData_t createImageData();
ImageData_t createBlankImage();
ImageData_t loadYZiFormImage();
ImageData_t loadBitmapImage();

//Drawing Stuff
window_t *initMainWindow(ImageData_t dImage);
window_t *initColorsWindow(color_t *color);
window_t *initToolsWindow();


//For Tools Window
void drawToolsWindow();
void drawCircleTool();
void drawFilledCircleTool();
void drawEllipseTool();
void drawFilledEllipseTool();
void drawTWSelectionBoxes(Tool_t *presTool);
void drawSelBox(int x, int row);

//For Colors Window
void drawColorsWindow(color_t *color);
void drawRed(color_t* color);
void drawGreen(color_t* color);
void drawBlue(color_t* color);
void drawMixed(color_t* color);

//Handling Stuff
void handleWindows(color_t* presColor, ImageData_t *image, window_t *main, window_t *colors, window_t *tools);
void handleToolsWindow(Tool_t *presTool);
void processToolClick(int x, int y, Tool_t *presTool);
bool handleColorsWindow(color_t* presColor);
bool processColorsClick(int x,int y, color_t *presColor);
void handleMainWindow(color_t* presColor, ImageData_t *image, Tool_t *presTool, window_t* colors);
bool handleMainDot(color_t* presColor, ImageData_t *image, int x, int y);
bool handleMainRect(color_t* presColor, ImageData_t *image, int x, int y);
bool handleMainLine(color_t* presColor, ImageData_t *image, int x, int y);
bool handleMainDropper(color_t* presColor, ImageData_t *image, window_t* colors, int x, int y);
bool handleBucket(color_t* presColor, ImageData_t *image, int x, int y);
bool handleCircle(color_t* presColor, ImageData_t *image, int x, int y);
bool handleFilledCircle(color_t *presColor, ImageData_t *image, int x, int y);
bool handleEllipse(color_t *presColor, ImageData_t *image, int x, int y);
bool handleFilledEllipse(color_t *presColor, ImageData_t *image, int x, int y);

//For Drawing & Demoing
void demoRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor);
void refreshRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor);
void onRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor, pixelFunction pf);
void demoLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor);
void refreshLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor);
void drawLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor);
void onLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor, pixelFunction pf);
void demoCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor);
void refreshCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor);
void drawCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor);
void onCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor, pixelFunction pf);
void drawFilledCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor);
void demoEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor);
void refreshEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor);
void drawEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor);
void onEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor, pixelFunction pf);
void drawFilledEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor);
void bucketFill(color_t origColor, color_t *presColor, ImageData_t *image, int x, int y, int rlim);
void addPixel(int x, int y, color_t *presColor, ImageData_t *image);
void refreshPixel(int x, int y, color_t *presColor, ImageData_t *image); //presColor is a dummy var
void demoPixel(int x, int y, color_t *presColor, ImageData_t *image);

//Other Misc
int	convertToColorNum(int x);
bool colorsEqual(color_t *a, color_t *b);

//Saving Stuff
void saveToFile(ImageData_t *image);
bool wantsToSave();
FILE* getFile();


main()
{
 	window_t *main_window, *colors_window, *tools_window;	
 	ImageData_t dImage;
 	color_t pres_color;
 	
 	dImage = createImageData();
 	main_window = initMainWindow(dImage);
 	colors_window = initColorsWindow(&pres_color);
 	tools_window = initToolsWindow();
 	handleWindows(&pres_color, &dImage, main_window, colors_window, tools_window);
 	yzDeleteWindow(main_window);
 	yzDeleteWindow(colors_window);
 	yzDeleteWindow(tools_window);
 	
 	saveToFile(&dImage);
}


ImageData_t createImageData()
{
	char text_buf[MAX_FN_LEN];
	
	printf("Would you like to\n");
	printf("1) Create A New Image\n");
	printf("2) Load a YZiForm Image\n");
	printf("3) Load a Bitmap Image\n");
	printf("Please Enter your choice > ");
	scanf("%s", text_buf);
	
	if(strlen(text_buf) != 1)
	{
		printf("\nPlease Enter 1, 2 or 3\n\n"); 
		return createImageData();
	}
	
	switch(text_buf[0])
	{
		case '1': return createBlankImage();
		case '2': return loadYZiFormImage();
		case '3': return loadBitmapImage();
		default: printf("\nPlease Enter 1, 2 or 3\n\n"); return createImageData();
	}
}

/* Function: ImageData_t createBlankImage()
 * Gets input from the user as to how to create the image and designs the 
 * ImageData_t it returns accordingly, allocating space and setting the 
 * width and height variables.
 */
ImageData_t createBlankImage()
{
	ImageData_t dImage;
	int i, j;
	
	printf("Please enter image width: ");
	scanf("%d", &(dImage.width));
	printf("Please enter image height: ");
	scanf("%d", &(dImage.height));
	
	if(dImage.width > MAX_WIDTH || dImage.height > MAX_HEIGHT)
	{
		printf("Height or Width too big\n");
		printf("Please limit width to %d and height to %d and try again\n\n"
			  , MAX_WIDTH, MAX_HEIGHT);
		
		return createImageData();
	}
	
	assert(dImage.image_data = malloc(dImage.height * sizeof(color_t*)));
	
	for(i=0; i < dImage.height; i++)
	{
		assert(dImage.image_data[i] = malloc(dImage.width * sizeof(color_t)));
		
		assert(dImage.image_data[i]);
		
		for(j=0; j < dImage.width; j++)
		{
			dImage.image_data[i][j].r= dImage.image_data[i][j].g=dImage.image_data[i][j].b = 65535;
			dImage.image_data[i][j].a = 0;
		}
	}
	
	return dImage;
}


//Loads a YZiForm Image in an ImageData_t
ImageData_t loadYZiFormImage()
{
	char t_buf[MAX_FN_LEN];
	FILE *infile;
	yzImage inImage;
	ImageData_t dImage;
	
	printf("Please enter the filename > ");
	scanf("%s", t_buf);
	
	infile = fopen(t_buf, "r");
	
	if(!infile)
	{
		printf("\nCannot open file %s, please try again\n\n", t_buf);
		return loadYZiFormImage();
	}
	
	inImage = openImageFile(infile);
	fclose(infile);
	
	dImage.image_data = toColors(inImage);
	dImage.width = getImageWidth(inImage);
	dImage.height = getImageHeight(inImage);
	destroyImage(inImage);
	return dImage;
}


//Load a Bitmap Image
ImageData_t loadBitmapImage()
{
	ImageData_t dImage;
	FILE *infile;
	char t_buf[MAX_FN_LEN];
	
	printf("Please enter the filename > ");
	scanf("%s", t_buf);
	
	infile = fopen(t_buf, "r");
	
	if(!infile)
	{
		printf("\nCannot open file %s, please try again\n\n", t_buf);
		return loadBitmapImage();
	}
	
	dImage.image_data = readBMPFile(infile, &(dImage.width), &(dImage.height));
	fclose(infile);
	
	return dImage;
}


/* Function: window_t *initMainWindow(ImageData_t dImage)
 * This function inits the MainWindow by setting its size according to the 
 * ImageData_t, setting its background color, moving it and selecting it. 
 * It returns the window_t* for the main window.
 */
window_t *initMainWindow(ImageData_t dImage)
{
	window_t *dWindow;
	
	dWindow=yzNewWindow(WINDOW_NAME, dImage.width * PIXEL_SIZE, dImage.height* PIXEL_SIZE);
	yzSelectWindow(dWindow);
	yzPenColor(65535, 65535, 65535);
	
	MacMoveWindow(FrontWindow(), 20+ CW_WIDTH, 40, 1); //Mac Specific
	
	return dWindow;
}


/* Function: window_t *initColorsWindow(color_t *color)
 * This function inits the color window, draws it contents and moves it 
 * into place. It returns the window_t* for the color window.
 */
window_t *initColorsWindow(color_t *color)
{
	window_t *cWindow;
	
	cWindow = yzNewWindow(CW_NAME, CW_WIDTH, CW_HEIGHT);
	yzSelectWindow(cWindow);
	
	color->r = color->g = color->b = 0;
	drawColorsWindow(color);
	
	MacMoveWindow(FrontWindow(), 5, 40, 1);//Mac Specific
	
	return cWindow;
}


/* Function: window_t *initToolsWindow()
 * This function inits the tools window by creating the window and 
 * drawing its contents. It returns the window_t* for the tools window.
 */
window_t *initToolsWindow()
{
	window_t *bWindow;
	
	bWindow = yzNewWindow(TW_NAME, TW_WIDTH, TW_HEIGHT);
	yzSelectWindow(bWindow);
	
	drawToolsWindow();
	
	MacMoveWindow(FrontWindow(), 5, 40 + CW_HEIGHT + 30, 1); //Mac Specific
	
	return bWindow;
}


/* Function: void drawToolsWindow()
 * Draws the boxes that make up the tools window.
 */
void drawToolsWindow()
{
	FILE *toolsin;
	yzImage bucket;
	
	assert(toolsin = fopen(TW_FILENAME, "r"));
	bucket = openImageFile(toolsin);
	fclose(toolsin);

	yzPenColor(65535*.75, 65535*.75, 65535*.75);
	yzRect(0,0,TW_WIDTH,TW_HEIGHT);
	
	//Draw DivisionsTwo Boxes only
	yzPenColor(0,0,0);
	yzLine(TW_BOX_H_SPACER, 0, TW_BOX_H_SPACER, TW_HEIGHT);
	yzLine(TW_BOX_H_SPACER*2, 0, TW_BOX_H_SPACER*2, TW_HEIGHT);
	
	//Draw Row Divisions
	yzLine(0, TW_ROW2_TOP, TW_WIDTH, TW_ROW2_TOP);
	yzLine(0, TW_ROW3_TOP, TW_WIDTH, TW_ROW3_TOP);
	
	//Draw Dot Tool
	yzRect(TW_DOT_START, TW_DOT_START, TW_DOT_END, TW_DOT_END);
	drawSelBox(TW_SELECT_BUF, 1);
	
	//Draw Rect Tool
	yzRect(TW_RECT_H_START, TW_TOOL_START, TW_RECT_H_END, TW_TOOL_END);
	
	//Draw Line Tool
	yzLine(TW_LINE_H_START, TW_TOOL_START, TW_LINE_H_END, TW_TOOL_END);
	
	//Draw Dropper Tool
	yzPenColor(0,0,0);
	yzLine(TW_TOOL_START, (TW_ROW2_START + TW_ROW2_END)/2, TW_TOOL_END, (TW_ROW2_START + TW_ROW2_END)/2);
	yzLine((TW_TOOL_START + TW_TOOL_END)/2, TW_ROW2_START, (TW_TOOL_START + TW_TOOL_END)/2, TW_ROW2_END);
	
	//Draw Bucket Tool
	drawImage(bucket, TW_RECT_H_START, TW_ROW2_START);
	destroyImage(bucket);
	
	//Draw Circle Tool
	drawCircleTool();
	
	//Draw Filled Circle Tool
	drawFilledCircleTool();
	
	//Draw Ellipse Tool
	drawEllipseTool();
	
	//Draw Filled Ellipse Tool
	drawFilledEllipseTool();
}


//Draws the Circle Tool for the Tools Window
void drawCircleTool()
{
	int x, y;
	color_t black;
	
	black.r=black.g=black.b=0;
	
	yzPixel(TW_CIRCLE_CX + TW_CIRCLE_RAD -1, TW_CIRCLE_CY, black);
	yzPixel(TW_CIRCLE_CX - TW_CIRCLE_RAD +1, TW_CIRCLE_CY, black);
	yzPixel(TW_CIRCLE_CX, TW_CIRCLE_CY + TW_CIRCLE_RAD -1, black);
	yzPixel(TW_CIRCLE_CX, TW_CIRCLE_CY - TW_CIRCLE_RAD +1, black);
	
	for(x=1; x < TW_CIRCLE_RAD; x++)
	{
		y = sqrt(TW_CIRCLE_RAD*TW_CIRCLE_RAD-x*x);
		
		yzPixel(TW_CIRCLE_CX+x, TW_CIRCLE_CY+y, black);
		yzPixel(TW_CIRCLE_CX-x, TW_CIRCLE_CY+y, black);
		yzPixel(TW_CIRCLE_CX+x, TW_CIRCLE_CY-y, black);
		yzPixel(TW_CIRCLE_CX-x, TW_CIRCLE_CY-y, black);
		yzPixel(TW_CIRCLE_CX+y, TW_CIRCLE_CY+x, black);
		yzPixel(TW_CIRCLE_CX-y, TW_CIRCLE_CY+x, black);
		yzPixel(TW_CIRCLE_CX+y, TW_CIRCLE_CY-x, black);
		yzPixel(TW_CIRCLE_CX-y, TW_CIRCLE_CY-x, black);
	}
}

//Draws the Filled Circle Tool
void drawFilledCircleTool()
{
	int i, j;
	
	yzPenColor(0,0,0);
	
	yzLine(TW_FCIRC_CX - TW_CIRCLE_RAD +1, TW_FCIRC_CY, TW_FCIRC_CX + TW_CIRCLE_RAD -1, TW_FCIRC_CY);
	yzLine(TW_FCIRC_CX, TW_FCIRC_CY - TW_CIRCLE_RAD +1, TW_FCIRC_CX, TW_FCIRC_CY + TW_CIRCLE_RAD-1);
	
	for(i=1; i < TW_CIRCLE_RAD; i++)
	{
		j = sqrt(TW_CIRCLE_RAD*TW_CIRCLE_RAD - i * i);
		
		yzLine(i+TW_FCIRC_CX,j+TW_FCIRC_CY,-i+TW_FCIRC_CX,j+TW_FCIRC_CY);
		yzLine(i+TW_FCIRC_CX,-j+TW_FCIRC_CY,-i+TW_FCIRC_CX, -j+TW_FCIRC_CY);
		yzLine(j+TW_FCIRC_CX,i+TW_FCIRC_CY,-j+TW_FCIRC_CX,i+TW_FCIRC_CY);
		yzLine(j+TW_FCIRC_CX,-i+TW_FCIRC_CY,-j+TW_FCIRC_CX,-i+TW_FCIRC_CY);
	}
}


//Draws the Ellipse Tool
void drawEllipseTool()
{
	double x, y, a, b;
	color_t black;
	
	black.r = black.g = black.b = 0;
	
	a = TW_ELPSE_WID;
	b = TW_ELPSE_HGT;
	
	for(x=0; x <= a; x++)
	{
		y = b*sqrt(1-x*x/(a*a));
		
		yzPixel(x+TW_ELPSE_CX,y+TW_FCIRC_CY,black);
		yzPixel(-x+TW_ELPSE_CX,y+TW_FCIRC_CY,black);
		yzPixel(x+TW_ELPSE_CX,-y+TW_FCIRC_CY,black);
		yzPixel(-x+TW_ELPSE_CX,-y+TW_FCIRC_CY, black);
	}
	
	for(y=0; y <= b; y++)
	{
		x = a * sqrt(1-y*y/(b*b));
		
		yzPixel(x+TW_ELPSE_CX,y+TW_FCIRC_CY,black);
		yzPixel(-x+TW_ELPSE_CX,y+TW_FCIRC_CY,black);
		yzPixel(x+TW_ELPSE_CX,-y+TW_FCIRC_CY,black);
		yzPixel(-x+TW_ELPSE_CX,-y+TW_FCIRC_CY, black);
	}
}


void drawFilledEllipseTool()
{
	double x, y, a, b;
	
	a = TW_ELPSE_WID;
	b = TW_ELPSE_HGT;
	
	yzPenColor(0,0,0);
	
	for(x=0; x <= a; x++)
	{
		y = b*sqrt(1-x*x/(a*a));
		
		yzLine(x+TW_CIRCLE_CX, TW_FCIRC_CY + y,TW_CIRCLE_CX-x, TW_FCIRC_CY+y);
		yzLine(x+TW_CIRCLE_CX, TW_FCIRC_CY - y,TW_CIRCLE_CX-x, TW_FCIRC_CY-y);
	}
	
	for(y=0; y <= b; y++)
	{
		x = a * sqrt(1-y*y/(b*b));
		
		yzLine(x+TW_CIRCLE_CX, TW_FCIRC_CY + y,TW_CIRCLE_CX-x, TW_FCIRC_CY+y);
		yzLine(x+TW_CIRCLE_CX, TW_FCIRC_CY - y,TW_CIRCLE_CX-x, TW_FCIRC_CY-y);
	}
}


/* Function: void drawTWSelectionBoxes(Tool_t *presTool)
 * This function draws (and erases) the selection boxes for the
 * tools window. The selected box is the one passed in.
 */
void drawTWSelectionBoxes(Tool_t *presTool)
{
	int i;
	
	yzPenColor(65535*.75, 65535*.75, 65535*.75);
		
	//Overwrites old boxes
	for(i=0; i < TW_NUM_V_BOXES; i++)
	{
		drawSelBox(TW_SELECT_BUF,i+1);
		drawSelBox(TW_RECT_H_START - BUF_DIF, i+1);
		drawSelBox(TW_LINE_H_START - BUF_DIF, i+1);
	}
	
	yzPenColor(0,0,0);
	
	switch(*presTool)
	{
	 	case dot: 		drawSelBox(TW_SELECT_BUF, 1); 				break;
	 	case rect:		drawSelBox(TW_RECT_H_START - BUF_DIF, 1);	break;
	 	case line:		drawSelBox(TW_LINE_H_START - BUF_DIF, 1);	break;
	 	case dropper:	drawSelBox(TW_SELECT_BUF, 2);				break;
	 	case bucket:	drawSelBox(TW_RECT_H_START - BUF_DIF, 2);	break;
	 	case circle: 	drawSelBox(TW_LINE_H_START - BUF_DIF, 2);	break;
	 	case fillcirc:  drawSelBox(TW_SELECT_BUF,3);				break;
	 	case ellipse:	drawSelBox(TW_RECT_H_START - BUF_DIF,3);	break;
	 	case fellipse:	drawSelBox(TW_LINE_H_START - BUF_DIF, 3);	break;
	 	default:													break;
	}
}



/* Function: void drawSelBox(int x)
 * Draws an individual selection box. Doesn't set color. 
 * NOTE: Prototype may change soon. DON'T FORGET to update comment.
 */
void drawSelBox(int x, int row)
{
	row --;
	
	yzLine(x, TW_SELECT_BUF + row * TW_ROW2_TOP, x + TW_SELECT_SIDE, TW_SELECT_BUF+ row * TW_ROW2_TOP);
	yzLine(x, TW_SELECT_BTM+ row * TW_ROW2_TOP, x + TW_SELECT_SIDE, TW_SELECT_BTM+ row * TW_ROW2_TOP);
	yzLine(x, TW_SELECT_BUF+ row * TW_ROW2_TOP, x, TW_SELECT_BTM+ row * TW_ROW2_TOP);
	yzLine(x + TW_SELECT_SIDE, TW_SELECT_BUF+ row * TW_ROW2_TOP, x + TW_SELECT_SIDE, TW_SELECT_BTM+ row * TW_ROW2_TOP);
}


/* Function: void drawColorsWindow(color_t *color)
 * This function draws the colors window. The color_t* represents
 * the colors for the palettes and the main color.
 */
void drawColorsWindow(color_t *color)
{
	yzPenColor(65535*.75, 65535*.75, 65535*.75);
	yzRect(0,0,CW_WIDTH,CW_HEIGHT);
	
	//Draw Exit Box
	yzPenColor(0,0,0);
	yzRect(CW_WIDTH - CW_EXIT_SIDE, CW_HEIGHT - CW_EXIT_SIDE, CW_WIDTH, CW_HEIGHT);
	yzPenColor(65535*.5, 65535*.5, 65536*.5);
	yzLine(CW_WIDTH - CW_EXIT_SIDE, CW_HEIGHT - CW_EXIT_SIDE, CW_WIDTH, CW_HEIGHT);
	yzLine(CW_WIDTH - CW_EXIT_SIDE, CW_HEIGHT, CW_WIDTH, CW_HEIGHT - CW_EXIT_SIDE);
	
	//Draw Transparency Box
	yzPenColor(0,0,0);
	yzRect(0, CW_HEIGHT - CW_EXIT_SIDE, CW_EXIT_SIDE, CW_HEIGHT);
	yzPenColor(65535*.5, 65535*.5, 65536*.5);
	yzLine(CW_EXIT_SIDE /2, CW_HEIGHT - CW_EXIT_SIDE +CW_TRANS_BUF, CW_EXIT_SIDE /2 , CW_HEIGHT -CW_TRANS_BUF);
	yzLine(CW_TRANS_BUF, CW_HEIGHT - CW_EXIT_SIDE +CW_TRANS_BUF, CW_EXIT_SIDE -CW_TRANS_BUF-1, CW_HEIGHT - CW_EXIT_SIDE +CW_TRANS_BUF);
	
	if(color->a != TRANS_A)
	{
		drawRed(color);
		drawGreen(color);
		drawBlue(color);
	}
}


/* Function: void drawRed(color_t* color)
 * Draws the red box of the tools window.
 */
void drawRed(color_t* color)
{
	int i;
	
	for(i=0; i < CW_COLOR_W; i++)
	{
		yzPenColor(65535 * i / CW_COLOR_W, 0,0);
		yzLine(CW_SIDE_BUFFER +i, CW_V_BUFFER, CW_SIDE_BUFFER +i, CW_V_BUFFER + CW_COLOR_H);
	}
	yzPenColor(0,0,0);
	yzLine(CW_SIDE_BUFFER+(color->r * 100) / 65535, CW_V_BUFFER,CW_SIDE_BUFFER+ (color->r * 100) / 65535, CW_V_BUFFER + CW_COLOR_H -1);
	drawMixed(color);
}


/* Function: void drawGreen(color_t* color)
 * Draws the green box of the tools window.
 */
void drawGreen(color_t* color)
{
	int i;
	
	for(i=0; i < CW_COLOR_W; i++)
	{
		yzPenColor(0, 65535 * i / CW_COLOR_W,0);
		yzLine(CW_SIDE_BUFFER +i, CW_V_BUFFER*2 + CW_COLOR_H, CW_SIDE_BUFFER +i, CW_V_BUFFER*2 + CW_COLOR_H*2);
	}
	yzPenColor(0,0,0);
	yzLine(CW_SIDE_BUFFER+(color->g * 100) / 65535, CW_V_BUFFER*2 + CW_COLOR_H, CW_SIDE_BUFFER+ (color->g * 100) / 65535, CW_V_BUFFER*2 + CW_COLOR_H*2 -1);
	drawMixed(color);
}


/* Function: void drawBlue(color_t* color)
 * Draws the blue box of the tools window.
 */
void drawBlue(color_t* color)
{
	int i;
	
	for(i=0; i < CW_COLOR_W; i++)
	{
		yzPenColor(0, 0,65535 * i / CW_COLOR_W);
		yzLine(CW_SIDE_BUFFER +i, CW_V_BUFFER*3 + CW_COLOR_H*2, CW_SIDE_BUFFER +i, CW_V_BUFFER*3 + CW_COLOR_H*3);
	}
	yzPenColor(0,0,0);
	yzLine(CW_SIDE_BUFFER+(color->b * 100) / 65535, CW_V_BUFFER*3 + CW_COLOR_H*2,CW_SIDE_BUFFER+ (color->b * 100) / 65535, CW_V_BUFFER*3 + CW_COLOR_H*3 -1);
	drawMixed(color);
}


/* Function: void drawMixed(color_t* color)
 * Draws the final color (mixed color) box of the tools window.
 */
void drawMixed(color_t* color)
{
	yzPenColor1c(*color);
	yzRect(CW_FINAL_SIDE, CW_V_BUFFER*4 + CW_COLOR_H*3, CW_FINAL_SIDE + CW_FINAL_W, CW_V_BUFFER*4 + CW_COLOR_H*3 + CW_FINAL_H);
}


/* Function: void handleWindows(color_t* presColor, ImageData_t *image, window_t *main, window_t *colors, window_t *tools)
 * This function flips through the various windows of the program and exits
 * if the tools window returns a false, indicating that the user has clicked
 * the exit box.
 */
void handleWindows(color_t* presColor, ImageData_t *image, window_t *main, window_t *colors, window_t *tools)
{
	Tool_t presTool = dot;
	int i, j;
	
	yzSelectWindow(main);
	
	//Draw Image if one was loaded
	for(i=0; i < image->height; i++)
		for(j=0; j < image->width; j++)
			refreshPixel(j*2,i*2,&(image->image_data[i][j]), image);
	
	while(1)
	{
		yzSelectWindow(tools);
		handleToolsWindow(&presTool);
		
		yzSelectWindow(main);
		handleMainWindow(presColor, image, &presTool, colors);
		
		yzSelectWindow(colors);
		if(!handleColorsWindow(presColor))
			return;
	}
}

 
/* Function: void handleToolsWindow(Tool_t *presTool)
 * Processes all clicks within the tools window. Returns
 * if mouse leaves window.
 */
void handleToolsWindow(Tool_t *presTool)
{
	int x, y, query;
	
	while(1)
	{
		query = yzQueryPointer(&x,&y);
		
		if(query == -1)
			return;
			
		if(query == 3)
			processToolClick(x, y, presTool);
	}
}


/* Function: void processToolClick(int x, int y, Tool_t *presTool)
 * This function checks to see which tool has been clicked based on
 * the x & y coords and sets the presTool variable and draws the 
 * selection box accordingly.
 */
void processToolClick(int x, int y, Tool_t *presTool)
{
	//check for dot
	if(x > TW_TOOL_BUFFER && x < TW_TOOL_END && y > TW_TOOL_BUFFER && y < TW_TOOL_END)
		*presTool = dot;
	
	//check for rect
	if(x > TW_RECT_H_START && x < TW_RECT_H_END && y > TW_TOOL_BUFFER && y < TW_TOOL_END)
		*presTool = rect;
	
	//check for line
	if(x > TW_LINE_H_START && x < TW_LINE_H_END && y > TW_TOOL_BUFFER && y < TW_TOOL_END)
		*presTool = line;
	
	//check for dropper
	if(x > TW_TOOL_BUFFER && x < TW_TOOL_END && y > TW_TOOL_BUFFER + TW_ROW2_TOP  && y < TW_TOOL_END + TW_ROW2_TOP)
		*presTool = dropper;
		
	//check for bucket
	if(x > TW_RECT_H_START && x < TW_RECT_H_END && y > TW_TOOL_BUFFER + TW_ROW2_TOP && y < TW_TOOL_END + TW_ROW2_TOP)
		*presTool = bucket;
		
	//check for circle
	if(x > TW_LINE_H_START && x < TW_LINE_H_END && y > TW_TOOL_BUFFER + TW_ROW2_TOP && y < TW_TOOL_END + TW_ROW2_TOP)
		*presTool = circle;
	
	//check for filled circle
	if(x > TW_TOOL_BUFFER && x < TW_TOOL_END && y > TW_TOOL_BUFFER + TW_ROW3_TOP && y < TW_TOOL_END + TW_ROW3_TOP)
		*presTool = fillcirc;
		
	//check for ellipse
	if(x > TW_RECT_H_START && x < TW_RECT_H_END && y > TW_TOOL_BUFFER + TW_ROW3_TOP && y < TW_TOOL_END + TW_ROW3_TOP)
		*presTool = ellipse;
	
	//check for filled ellipse
	if(x > TW_LINE_H_START && x < TW_LINE_H_END && y > TW_TOOL_BUFFER + TW_ROW3_TOP && y < TW_TOOL_END + TW_ROW3_TOP)
		*presTool = fellipse;
	
	drawTWSelectionBoxes(presTool);	
}


/* Function: bool handleColorsWindow(color_t* presColor)
 * Controls the colors window and returns false on exit click.
 */
bool handleColorsWindow(color_t* presColor)
{
	int x, y, query, orig;
	
	orig = clock();
	
	while(1)
	{
		query = yzQueryPointer(&x, &y);
		
		if(query == -1 && clock() - orig > 10)
			return true;
		if(query == 3 && !processColorsClick(x,y,presColor))
			return false;
	}
}


/* Function: bool processColorsClick(int x, int y, color_t *presColor)
 * Deals with clicks in the colors window. Returns false on exit
 */
bool processColorsClick(int x, int y, color_t *presColor)
{
	if(x > CW_SIDE_BUFFER && x < CW_WIDTH - CW_SIDE_BUFFER)
	{
		if(y > CW_V_BUFFER && y < CW_V_BUFFER + CW_COLOR_H)
		{
			presColor->r=convertToColorNum(x);
			drawRed(presColor);
		}
	
		if(y > CW_V_BUFFER*2 + CW_COLOR_H && y < CW_V_BUFFER*2 + CW_COLOR_H*2)
		{
			presColor->g=convertToColorNum(x);
			drawGreen(presColor);
		}
		
		if(y > CW_V_BUFFER*3 + CW_COLOR_H*2 && y < CW_V_BUFFER*3 + CW_COLOR_H*3)
		{
			presColor->b=convertToColorNum(x);
			drawBlue(presColor);
		}
	}
	
	//Transparency Suppport
	if(x > 0 && x < CW_EXIT_SIDE && y > CW_HEIGHT - CW_EXIT_SIDE)
	{
		presColor->a = (presColor->a == TRANS_A) ? (presColor->a - 1) : TRANS_A;
		drawColorsWindow(presColor);
	}
	
	//Check for Exit
	if(x > CW_WIDTH - CW_EXIT_SIDE && y > CW_HEIGHT - CW_EXIT_SIDE)
		return false;
		
	return true;
}



/* Function: void handleMainWindow(color_t* presColor, ImageData_t *image, Tool_t *presTool)
 * Processes all drags within the main window. Returns
 * if mouse leaves window.
 */
void handleMainWindow(color_t* presColor, ImageData_t *image, Tool_t *presTool, window_t* colors)
{
	int x, y, query;
	
	while(1)
	{
		query = yzQueryPointer(&x,&y);
		
		if(query == -1)
			return;
			
		if(query == 2 && x >= 0 && y >= 0 && x / PIXEL_SIZE < image->width && y / PIXEL_SIZE < image->height)
		{
			switch(*presTool)
			{
				case dot: 		if(!handleMainDot(presColor, image, x, y))
						  			return;
						  		break;
				case rect:		if(!handleMainRect(presColor, image, x, y))
									return; 	
								break;
				case line: 		if(!handleMainLine(presColor, image, x, y)) 	
									return;
								break;
				case dropper: 	if(!handleMainDropper(presColor, image, colors, x, y))
									return;
								break;
				case bucket: 	if(!handleBucket(presColor, image, x, y))
									return;
								break;
				case circle:	if(!handleCircle(presColor, image, x, y))
									return;
								break;
				case fillcirc: 	if(!handleFilledCircle(presColor, image, x,y))
									return;
								break;
				case ellipse:  	if(!handleEllipse(presColor, image, x, y))
									return;
								break;
				case fellipse:  if(!handleFilledEllipse(presColor, image, x, y))
									return;
								break;
				default:	break;
			}
		}
	}
}


/* Function: bool handleMainDot(color_t* presColor, ImageData_t *image, int x, int y)
 * Function for continuous dot drawing. Draws as long as the mouse button is down.
 */
bool handleMainDot(color_t* presColor, ImageData_t *image, int x, int y)
{
	int query, ox, oy;
	
	addPixel(x, y, presColor, image);
	
	while(1)
	{
		ox = x;
		oy = y;
		
		query = yzQueryPointer(&x, &y);
		
		if(query == 3)
			return true;
			
		if(query == -1)
			return false;
		
		if(query ==2 || query ==1)
			drawLine(x, y, ox, oy, image, presColor);
	}
}


/* Function: bool handleMainRect(color_t* presColor, ImageData_t *image, int x, int y)
 * Function for drawing rectangles. Draws an outline for the rectangle while the mouse
 * is down and draws the actual rectange once the mouse is released.
 */
bool handleMainRect(color_t* presColor, ImageData_t *image, int x, int y)
{
	int x1, y1, lx=0, ly=-1, query, i, j, clocked;
	bool flag;
	
	clocked = clock();
		
	while(1)
	{
		query=yzQueryPointer(&x1, &y1);
		
		flag = (query > 0);
		
		if(x1 >= 0 && y1 >= 0 && x1/PIXEL_SIZE < image->width && y1/PIXEL_SIZE < image->height)
		{
			if(flag && clock()-clocked > REDRAW_TIME)
			{
				if(ly != -1)
					refreshRectOutline(x, y, lx, ly, image, presColor);
					
				demoRectOutline(x, y, x1, y1, image, presColor);
				lx = x1; ly = y1;
				flag = false;
				clocked = clock();
			}
		
			//Erase ending outline
			if(query == -1)
			{
				refreshRectOutline(x, y, lx, ly, image, presColor);
				return false;
			}
			
			if(query == 3)
			{
				if(ly != -1)
					refreshRectOutline(x, y, lx, ly, image, presColor);
				break;
			}
		}
	}
	
	//swap to put x1 as larger x val
	if(x > x1)
	{
		i = x;
		x = x1;
		x1 = i;
	}
	
	//swap to put y1 as larger y val
	if(y > y1)
	{
		i = y;
		y = y1;
		y1 = i;
	}
	
	//Tries to fix artifacts problem
	x -= (x%PIXEL_SIZE);
	x1 += ((x1+1)%PIXEL_SIZE);
	y -= (y%PIXEL_SIZE);
	y1 +=((y1+1)%PIXEL_SIZE);
	
	//If Regular Color
	if(presColor->a != TRANS_A)
		yzPenColor1c(*presColor);
	//If Transparent
	else
		yzPenColor(65535*.75,65535*.75,65535*.75);
	
	yzRect(x,y,x1,y1);

	
	for(i=y/PIXEL_SIZE; i <= y1/PIXEL_SIZE; i++)
	{
		for(j=x/PIXEL_SIZE; j <= x1/PIXEL_SIZE; j++)
		{
			if(j >= 0 && i >= 0 && j < image->width && i < image->height)
				image->image_data[i][j] = *presColor;
		}
	}
	
	return true;
}


/* Function: bool handleMainLine(color_t* presColor, ImageData_t *image, int x, int y)
 * Draws demo lines to the mouse until mouseup. Then line becomes final.
 */
bool handleMainLine(color_t* presColor, ImageData_t *image, int x, int y)
{
	int x1, y1, lx, ly=-1, query=0, clocked;
	bool flag;
	
	clocked = clock();
	
	while(1)
	{
		query=yzQueryPointer(&x1, &y1);
		
		flag = (query > 0);
		
		if(flag && (clock() - clocked > REDRAW_TIME))
		{
			if(ly != -1)
				refreshLine(x, y, lx, ly, image, presColor);
			
			demoLine(x, y, x1, y1, image, presColor);
			lx = x1; ly = y1;
			flag = false;
			clocked = clock();
		}
		
		//Erase leading dot if end of drawing
		if(query == -1)
		{
			refreshLine(x, y, lx, ly, image, presColor);
			return false;
		}
		
		if(query == 3)
		{
			if(ly != -1)
				refreshLine(x, y, lx, ly, image, presColor);
			break;
		}
	}
	
	drawLine(x, y, x1, y1, image, presColor);
	return true;
}


//Handles Main Dropper
bool handleMainDropper(color_t* presColor, ImageData_t *image, window_t* colors, int x, int y)
{
	window_t *main_win;
	int query, clocked;
	color_t lastColor;
	
	clocked = clock();
	
	while(1)
	{
		if(clock()-clocked > REDRAW_TIME)
		{
			lastColor = *presColor;
			
			if(x >= 0 && y >= 0 && x/PIXEL_SIZE < image->width && y/PIXEL_SIZE < image->height)
				*presColor = image->image_data[y/2][x/2];
			
			if(!colorsEqual(presColor, &lastColor))
			{
				main_win = yzSelectWindow(colors);
				
				if(presColor->r != lastColor.r)
					drawRed(presColor);
				if(presColor->b != lastColor.b)
					drawBlue(presColor);
				if(presColor->g != lastColor.g)
					drawGreen(presColor);
				if((presColor->a == TRANS_A || lastColor.a != TRANS_A) && presColor->a != lastColor.a)
					drawColorsWindow(presColor);
					
				yzSelectWindow(main_win);
				clocked = clock();
			}
		}
			
		query=yzQueryPointer(&x,&y);
		
		if(query == -1)
			return false;
		if(query ==3)
			return true;	
	}
}


bool handleBucket(color_t* presColor, ImageData_t *image, int x, int y)
{
	int query;
	
	while(1)
	{
		query= yzQueryPointer(&x, &y);
		
		if(query ==3)
		{
			x/=PIXEL_SIZE; y/=PIXEL_SIZE;
			
			if(!colorsEqual(&(image->image_data[y][x]), presColor))
				bucketFill(image->image_data[y][x], presColor, image, x, y,0);
			
			return true;
		}
		
		if(query == -1)
			return false;
	}
}


//Handles Circle
bool handleCircle(color_t* presColor, ImageData_t *image, int x, int y)
{
	int nx, ny, nrad, lrad=0, query, clocked;
	bool flag;
	
	clocked = clock();
	
	while(1)
	{
		query = yzQueryPointer(&nx, &ny);
		nrad = sqrt(pow(abs(nx - x),2)+pow(abs(ny-y),2));
		
		if(query == 3)
		{
			refreshCircle(x, y, lrad, image, presColor);
			drawCircle(x, y, nrad, image, presColor);
			return true;
		}
		
		if(query == -1)
		{
			refreshCircle(x, y, lrad, image, presColor);
			return false;
		}
			
		flag = (query > 0);
		
		if(flag && (clock()-clocked  > REDRAW_TIME))
		{
			refreshCircle(x, y, lrad, image, presColor);
			demoCircle(x, y, nrad, image, presColor);
			lrad = nrad;
			clocked = clock();
			flag = false;
		}
	}
}


//Handles Filled circle tool
bool handleFilledCircle(color_t *presColor, ImageData_t *image, int x, int y)
{
	int nx, ny, nrad, lrad=0, query, clocked;
	bool flag;
	
	clocked = clock();
	
	while(1)
	{
		query = yzQueryPointer(&nx, &ny);
		nrad = sqrt(pow(abs(nx - x),2)+pow(abs(ny-y),2));
		
		if(query == 3)
		{
			refreshCircle(x, y, lrad, image, presColor);
			drawFilledCircle(x, y, nrad, image, presColor);
			return true;
		}
		
		if(query == -1)
		{
			refreshCircle(x, y, lrad, image, presColor);
			return false;
		}
		
		flag = (query>0);
			
		if(flag && (clock()-clocked > REDRAW_TIME))
		{
			refreshCircle(x, y, lrad, image, presColor);
			demoCircle(x, y, nrad, image, presColor);
			lrad = nrad;
			clocked = clock();
			flag = false;
		}
	}
}


//Handles clicks for ellipse function
bool handleEllipse(color_t *presColor, ImageData_t *image, int x, int y)
{
	int nx, ny, lx, ly, clocked, query;
	bool flag;
	
	clocked = clock();
	
	//Guard Against Leaving immediately
	lx = x;
	ly = y;
	
	while(1)
	{
		query = yzQueryPointer(&nx, &ny);
		
		flag = (query > 0);
		
		if(query == -1)
		{
			refreshEllipse(x, y, lx, ly, image, presColor);
			return false;
		}
		
		if(query == 3)
		{
			refreshEllipse(x, y, lx, ly, image, presColor);
			drawEllipse(x, y, nx, ny, image, presColor);
			return true;
		}
		
		if(flag && clock() - clocked > REDRAW_TIME)
		{
			refreshEllipse(x, y, lx, ly, image, presColor);
			demoEllipse(x, y, nx, ny, image, presColor);
			lx = nx;
			ly = ny;
			clocked = clock();
			flag = false;
		}
	}
}


//Filled Ellipse Handler
bool handleFilledEllipse(color_t *presColor, ImageData_t *image, int x, int y)
{
	int nx, ny, lx, ly, clocked, query;
	bool flag;
	
	clocked = clock();
	
	//Guard Against Leaving immediately
	lx = x;
	ly = y;
	
	while(1)
	{
		query = yzQueryPointer(&nx, &ny);
		
		flag = (query > 0);
		
		if(query == -1)
		{
			refreshEllipse(x, y, lx, ly, image, presColor);
			return false;
		}
		
		if(query == 3)
		{
			refreshEllipse(x, y, lx, ly, image, presColor);
			drawFilledEllipse(x, y, nx, ny, image, presColor);
			return true;
		}
		
		if(flag && clock() - clocked > REDRAW_TIME)
		{
			refreshEllipse(x, y, lx, ly, image, presColor);
			demoEllipse(x, y, nx, ny, image, presColor);
			lx = nx;
			ly = ny;
			clocked = clock();
			flag = false;
		}
	}
}


/* Function: void demoRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
 * Wrapper which draws "fake" rectangles which do not register in the image.
 */
void demoRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
{
	onRectOutline(x1, y1, x2, y2, image, presColor, (pixelFunction) demoPixel);
}


/* Function: void refreshRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
 * Wrapper which overwrites "fake" rects with actual contents.
 */
void refreshRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
{
	onRectOutline(x1, y1, x2, y2, image, presColor, (pixelFunction) refreshPixel);
}


/* Function: void onRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor, pixelFunction pf)
 * Draws rectangle outlines using the pixel drawing function supplied at the supplied coords in the supplied color.
 */
void onRectOutline(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor, pixelFunction pf)
{
	int i;
	
	if(x1 > x2)
	{
		i = x2; 
		x2 = x1;
		x1 = i;
	}
	
	if(y1 > y2)
	{
		i = y2; 
		y2 = y1;
		y1 = i;
	}
	
	for(i= x1; i <= x2; i+= PIXEL_SIZE)
	{
		pf(i, y2, presColor, image);
		pf(i, y1, presColor, image);
	}
	
	for(i= y1; i <= y2; i+= PIXEL_SIZE)
	{
		pf(x1, i, presColor, image);
		pf(x2, i, presColor, image);
	}
}


/* Function: void demoLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
 * This function draws a "fake" line.
 */
void demoLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
{
	onLine(x1, y1, x2, y2, image, presColor, (pixelFunction) demoPixel);
}


/* Function: void refreshLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
 * This function overwrites a "fake" line.
 */
void refreshLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
{
	onLine(x1, y1, x2, y2, image, presColor, (pixelFunction) refreshPixel);
}


/* Function: void refreshLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
 * This function creates a real line.
 */
void drawLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor)
{
	onLine(x1, y1, x2, y2, image, presColor, (pixelFunction) addPixel);
}


/* Function: void onLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor, pixelFunction pf)
 * A pixel for pixel line drawing function, uses the pixel drawing function supplied.
 */
void onLine(int x1, int y1, int x2, int y2, ImageData_t *image, color_t *presColor, pixelFunction pf)
{
	int i, j, largerx, smallerx, largery, smallery;
	double m;
	
	smallerx = (x1 > x2) ? x2 : x1;
	largerx = (x1 > x2) ? x1 : x2;
	smallery = (y1 > y2) ? y2 : y1;
	largery = (y1 > y2) ? y1 : y2;
	
	if(x1/PIXEL_SIZE == x2/PIXEL_SIZE)
	{
		for(j=smallery; j <=largery ; j+=2)
		{
			pf(x1, j, presColor, image);
		}
		
		return;
	}
	
	m = ((double)(y1) - (double)(y2))/((double)(x1) - (double)(x2));
		
	if(m >=0 && m <= 1)
	{
		for(i=0; i <= (largerx-smallerx); i+=2)
		{
			pf(i+smallerx, (i * m) + smallery, presColor, image);
		}
	}
	
	else if(m < 0 && m >= -1)
	{
		for(i=0; i <= (largerx-smallerx); i+=2)
		{
			pf(i+smallerx, (i * m) + largery, presColor, image);
		}
	}
	
	else if(m > 1)
	{
		m = 1/m;
		
		for(i=0; i <= (largery-smallery); i+=2)
		{
			pf((i*m)+smallerx, (i) + smallery, presColor, image);
		}
	}
	
	else if(m < -1)
	{
		m = 1/m;
		
		for(i=0; i <= (largery-smallery); i+=2)
		{
			pf((i*m)+largerx, (i) + smallery, presColor, image);
		}
	}
}


void demoCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor)
{
	onCircle(x,y,rad,image,presColor, (pixelFunction) demoPixel);
}


void refreshCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor)
{
	onCircle(x,y,rad,image,presColor, (pixelFunction) refreshPixel);
}


void drawCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor)
{
	onCircle(x,y,rad,image,presColor, (pixelFunction) addPixel);
}


void onCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor, pixelFunction pf)
{
	int i, j;
	
	pf(x + rad - 1, y, presColor, image);
	pf(x - rad + 1, y, presColor, image);
	pf(x, y + rad - 1, presColor, image);
	pf(x, y - rad + 1, presColor, image);
	
	for(i=1; i < rad; i++)
	{
		j = sqrt(rad*rad-i*i);
		
		pf(x+i, y+j, presColor, image);
		pf(x-i, y+j, presColor, image);
		pf(x+i, y-j, presColor, image);
		pf(x-i, y-j, presColor, image);
		pf(x+j, y+i, presColor, image);
		pf(x-j, y+i, presColor, image);
		pf(x+j, y-i, presColor, image);
		pf(x-j, y-i, presColor, image);
	}
}


//Draws Filled Circle
void drawFilledCircle(int x, int y, int rad, ImageData_t *image, color_t *presColor)
{
	int i, j;
	
	drawLine(x - rad +1, y, x + rad -1, y, image, presColor);
	drawLine(x, y - rad +1, x, y + rad-1, image, presColor);
	
	for(i=1; i < rad; i++)
	{
		j = sqrt(rad*rad - i * i);
		
		drawLine(i+x,j+y,-i+x,j+y, image, presColor);
		drawLine(i+x,-j+y,-i+x, -j+y, image, presColor);
		drawLine(j+x,i+y,-j+x,i+y, image, presColor);
		drawLine(j+x,-i+y,-j+x,-i+y, image, presColor);
	}
}


void demoEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor)
{
	onEllipse(cx, cy, x1, y1, image, presColor, (pixelFunction) demoPixel);
}


void refreshEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor)
{
	onEllipse(cx, cy, x1, y1, image, presColor, (pixelFunction) refreshPixel);
}


void drawEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor)
{
	onEllipse(cx, cy, x1, y1, image, presColor, (pixelFunction) addPixel);
}


//Carries out pf on ellipse
void onEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor, pixelFunction pf)
{
	double a, b, x, y;
	
	a = abs(x1-cx);
	b = abs(y1-cy);
	
	for(x=0; x <= a; x++)
	{
		y = b*sqrt(1-(x*x)/(a*a));
		
		pf(x+cx, y+cy, presColor, image);
		pf(-x+cx, y+cy, presColor, image);
		pf(x+cx, -y+cy, presColor, image);
		pf(-x+cx, -y+cy, presColor, image);
	}
	
	for(y=0; y <= b; y++)
	{
		x = a * sqrt(1-(y*y)/(b*b));
		
		pf(x+cx, y+cy, presColor, image);
		pf(-x+cx, y+cy, presColor, image);
		pf(x+cx, -y+cy, presColor, image);
		pf(-x+cx, -y+cy, presColor, image);
	}
}


//Draws a Filled Ellipse
void drawFilledEllipse(int cx, int cy, int x1, int y1, ImageData_t *image, color_t *presColor)
{
	double a, b, x, y;
	
	a = abs(x1-cx);
	b = abs(y1-cy);
	
	for(x=0; x <= a; x++)
	{
		y = b*sqrt(1-(x*x)/(a*a));
		
		drawLine(cx-x,y+cy,x+cx, y+cy, image, presColor);
		drawLine(cx-x,cy-y,x+cx, cy-y, image, presColor);
	}
	
	for(y=0; y <= b; y++)
	{
		x = a * sqrt(1-(y*y)/(b*b));
		
		drawLine(cx-x,y+cy,x+cx, y+cy, image, presColor);
		drawLine(cx-x,cy-y,x+cx, cy-y, image, presColor);
	}
}


//Takes x/PIXEL_SIZE, y/PIXEL_SIZE, rlim to prevent infinite recursion
void bucketFill(color_t origColor, color_t* presColor, ImageData_t *image, int x, int y, int rlim)
{
	int i, j;

	if(rlim > RECURSE_LIM)
		return;
	if(x < 0 || y < 0 || x >= image->width || y >= image->height || !colorsEqual(&origColor, &(image->image_data[y][x])))
		return;
	
	rlim ++;
	
	for(i=0; (x+i) < image->width && colorsEqual(&origColor, &(image->image_data[y][x+i])); i++)
		addPixel((x+i) *PIXEL_SIZE, y *PIXEL_SIZE, presColor, image);
	
	for(j=1; (x-j) >=0 && colorsEqual(&origColor, &(image->image_data[y][x-j])); j++)
		addPixel((x-j) *PIXEL_SIZE, y *PIXEL_SIZE, presColor, image);	
	
	for(i--; i>=0; i--)
	{	
		if(y+1 < image->height && colorsEqual(&origColor, &(image->image_data[y+1][x+i])))
			bucketFill(origColor, presColor, image, x+i, y+1,rlim);
		if(y-1 >=0 && colorsEqual(&origColor, &(image->image_data[y-1][x+i])))
			bucketFill(origColor, presColor, image, x+i, y-1, rlim);
	}
	
	//i=1 to avoid dying at start
	for(j--; j>0 ; j--)
	{
		if(y+1 < image->height && colorsEqual(&origColor, &(image->image_data[y+1][x-j])))
			bucketFill(origColor, presColor, image, x-j, y+1, rlim);
		if(y-1 >=0 && colorsEqual(&origColor, &(image->image_data[y-1][x-j])))
			bucketFill(origColor, presColor, image, x-j, y-1, rlim);
	}
}


/* Function: void addPixel(int x, int y, color_t *presColor, ImageData_t *image)
 * Draws and adds a pixel of presColor in the image.
 */
void addPixel(int x, int y, color_t *presColor, ImageData_t *image)
{
	x /=PIXEL_SIZE; y/=PIXEL_SIZE;
	
	if(x < 0 || y < 0 || x >= image->width || y >= image->height)
		return;
		
	image->image_data[y][x] = *presColor;
	
	//If Regular Color
	if(presColor->a != TRANS_A)
		yzPenColor1c(*presColor);
	//If Transparent
	else
	{
		yzPenColor(65535*.75,65535*.75,65535*.75);
		image->image_data[y][x].r = image->image_data[y][x].g = image->image_data[y][x].b = 0;
	}

	yzRect(x*PIXEL_SIZE, y*PIXEL_SIZE, (x+1)*PIXEL_SIZE-1, (y+1)*PIXEL_SIZE-1);
}


/* Function: void refreshPixel(int x, int y, color_t* presColor, ImageData_t *image)
 * This function rewrites the pixel at the given coords.
 */

void refreshPixel(int x, int y, color_t* presColor, ImageData_t *image)
{
	x /=PIXEL_SIZE; y /= PIXEL_SIZE;
	
	presColor=presColor; // To Avoid Warnings about presColor not being used (dummy variable)
	
	if(x < 0 || y < 0 || x >= image->width || y >= image->height)
		return;
	
	//If Regular	
	if(image->image_data[y][x].a != TRANS_A)
		yzPenColor1c(image->image_data[y][x]);
	//If Transparent
	else
		yzPenColor(65535*.75,65535*.75,65535*.75);
		
	yzRect(x *PIXEL_SIZE, y*PIXEL_SIZE, (x+1) * PIXEL_SIZE-1, (y+1) * PIXEL_SIZE-1);
}

 
/* Function: void demoPixel(int x, int y, color_t *presColor, ImageData_t *image)
 * Draws the box for the pixel on the screen without registering. Used in certain
 * drawing functions.
 */
void demoPixel(int x, int y, color_t *presColor, ImageData_t *image)
{
	x /=PIXEL_SIZE; y/=PIXEL_SIZE;
	
	if(x < 0 || y < 0 || x >= image->width || y >= image->height)
		return;
	
	//If Regular Color
	if(presColor->a != TRANS_A)
		yzPenColor1c(*presColor);
	//If Transparent
	else
		yzPenColor(65535*.75,65535*.75,65535*.75);
		
	yzRect(x*PIXEL_SIZE, y*PIXEL_SIZE, (x+1)*PIXEL_SIZE-1, (y+1)*PIXEL_SIZE-1);
}

/* Function: int convertToColorNum(int x)
 * Converts an x-coord to a number between 0 and 65535 based on its location
 */
int	convertToColorNum(int x)
{
	x-= CW_SIDE_BUFFER;
	return x * 65536 /100;
}


//Returns true if the colors are same else false/includes transparency support
bool colorsEqual(color_t *a, color_t *b)
{
	if(a->a == TRANS_A && b->a == TRANS_A)
		return true;
	
	if((a->a == TRANS_A && b->a != TRANS_A) || (a->a !=TRANS_A && b->a == TRANS_A))
		return false;
	
	return(a->r==b->r && a->b == b->b && a->g == b->g);
}


//Saving stuff
void saveToFile(ImageData_t *image)
{
	FILE *out;
	char t_buf[MAX_FN_LEN];
	
	if(!wantsToSave())
		return;
	
	printf("Would you like to save as a 1) YZiForm File or 2) Bitmap File > ");
	scanf("%s", t_buf);
	
	if(!strcmp(t_buf, "1"))
	{
		out = getFile();
		colorArrtoImageFile(image->width, image->height, image->image_data, out);
		fclose(out);
	}
	else if(!strcmp(t_buf, "2"))
	{
		out = getFile();
		colorArrtoBMP(image->image_data, out, image->width, image->height);
		fclose(out);
	}
	else
	{
		printf("\nPlease enter 1 or 2 it really isn't all that hard\n\n");
		saveToFile(image);
	}
}


bool wantsToSave()
{
	char save[MAX_FN_LEN];
	
	while(1)
	{
		printf("Do you want to save (y or n) > ");
		scanf("%s", save);
	
		if(!strcmp(save, "n") || !strcmp(save, "N"))
			return false;
		if(!strcmp(save, "y") || !strcmp(save, "Y"))
			return true;
			
		printf("Please enter either a y or n\n");
	}
}


FILE* getFile()
{
	char name[MAX_FN_LEN], proceed;
	
	printf("Please enter a filename> ");
	scanf("%s", name);
	
	printf("\n You have choosen \"%s\" as the filename.\n", name);
	printf("Do you wish to proceed [y or n] > ");
	scanf("\n%c", &proceed);
	
	if(proceed == 'Y' || proceed == 'y')
		return fopen(name, "w");
		
	else
	{
		printf("\n");
		return getFile();
	}
}