/* Steven Stanek
 * June 10, 2001
 * YZTetris
 */

#include "yztetris.h"
#include "tetrisfilereader.h"
#undef 	 feof

//Init Stuff
window_t* initWindow();
pieces_t* initPieces();
void drawGameArea();

//Command Stuff
void mainLoop();
bool handleActiveLevel(level_t *aLevel, pieces_t *allPieces, yzImage *deathImage);
void createRandomBlock(block_t *activeBlock, pieces_t *dPieces);
void moveBlockLeft(block_t *dBlock);
void moveBlockRight(block_t *dBlock);
void moveBlockDown(block_t *dBlock);
void rotateBlock(block_t *dBlock);

//Deals with Row Completion
void checkForRows(level_t *dLevel);
bool rowComplete(row_t *dRow);
void removeRow(level_t *aLevel, int level_num);
bool levelComplete(level_t *aLevel);
bool playerDied(level_t *aLevel);

//Drawing bricks in Window
void drawBrickAt(color_t *dColor, int x, int y); //ints x and y are in bricks, not pixels
void drawBrick(brick_t *dBrick);
void drawBlankAt(int x, int y);
void drawBlock(block_t *dBlock);
void drawAll(level_t *aLevel);
void refreshBrickAt(int x, int y);
void refreshBrick(brick_t *dBrick);
void refreshBlock(block_t *dBlock);

//Misc
void drawSideImage(yzImage aImage);
void blockToBricks(block_t *dBlock, level_t *allBricks);
bool blockCollided(block_t *dBlock, level_t *allBricks);
bool hasHitBottom(block_t *dBlock, level_t *allBricks);
void freeUpGame(game_t *dGame);
void doFinalImage(yzImage dImage);


main()
{
	window_t *main_window;
	
	srand(clock());
	
	main_window = initWindow();	
	mainLoop();
	
	return 0;
}


window_t *initWindow()
{
	window_t *main_window;
	
	main_window = yzNewWindow(WINDOW_NAME, WINDOW_WID, WINDOW_HGT);
	yzSelectWindow(main_window);
		
	return main_window;
}


pieces_t* initPieces(tetris_colors_t *allColors)
{
	pieces_t *dPieces;
	int i;
	
	dPieces = malloc(sizeof(pieces_t));
	
	for(i=0; i < BRICKS_IN_BLOCK; i++)
	{
		dPieces->longy.arrangement[i].dColor = allColors->blue;
		dPieces->trileft.arrangement[i].dColor = allColors->green;
		dPieces->triright.arrangement[i].dColor = allColors->red;
		dPieces->trimid.arrangement[i].dColor = allColors->orange;
		dPieces->bileft.arrangement[i].dColor = allColors->purple;
		dPieces->biright.arrangement[i].dColor = allColors->yellow;
		dPieces->square.arrangement[i].dColor = allColors->brown;
	}
	
	//Do Longy
	dPieces->longy.arrangement[0].y = START_LY -3;
	dPieces->longy.arrangement[1].y = START_LY -2;
	dPieces->longy.arrangement[2].y = START_LY -1;
	dPieces->longy.arrangement[3].y = START_LY;
	dPieces->longy.arrangement[0].x = dPieces->longy.arrangement[1].x 
		= dPieces->longy.arrangement[2].x= dPieces->longy.arrangement[3].x = START_LX;
	
	//Do TriLeft
	dPieces->trileft.arrangement[0].y = dPieces->trileft.arrangement[1].y 
		= dPieces->trileft.arrangement[2].y = START_LY -1;
	dPieces->trileft.arrangement[3].y = START_LY;
	dPieces->trileft.arrangement[0].x = START_LX - 1;
	dPieces->trileft.arrangement[1].x = START_LX;
	dPieces->trileft.arrangement[2].x = dPieces->trileft.arrangement[3].x  = START_LX + 1;
	
	//Do Triright
	dPieces->triright.arrangement[0].y = dPieces->triright.arrangement[1].y
		= dPieces->triright.arrangement[2].y = START_LY -1;
	dPieces->triright.arrangement[3].y = START_LY;
	dPieces->triright.arrangement[0].x = START_LX + 1;
	dPieces->triright.arrangement[1].x = START_LX;
	dPieces->triright.arrangement[2].x = dPieces->triright.arrangement[3].x  = START_LX - 1;
	
	//Do Trimiddle
	dPieces->trimid.arrangement[0].y = dPieces->trimid.arrangement[1].y
		= dPieces->trimid.arrangement[2].y = START_LY -1;
	dPieces->trimid.arrangement[3].y = START_LY;
	dPieces->trimid.arrangement[0].x = START_LX - 1;
	dPieces->trimid.arrangement[1].x = START_LX + 1 ;
	dPieces->trimid.arrangement[2].x = dPieces->trimid.arrangement[3].x  = START_LX;
	
	//Do Bileft
	dPieces->bileft.arrangement[0].y = START_LY;
	dPieces->bileft.arrangement[1].y = dPieces->bileft.arrangement[2].y = START_LY -1;
	dPieces->bileft.arrangement[3].y = START_LY -2;
	dPieces->bileft.arrangement[1].x = dPieces->bileft.arrangement[0].x = START_LX + 1;
	dPieces->bileft.arrangement[2].x = dPieces->bileft.arrangement[3].x = START_LX;
	
	//Do Biright
	dPieces->biright.arrangement[0].y = START_LY;
	dPieces->biright.arrangement[1].y = dPieces->biright.arrangement[2].y = START_LY -1;
	dPieces->biright.arrangement[3].y = START_LY -2;
	dPieces->biright.arrangement[1].x = dPieces->biright.arrangement[0].x = START_LX;
	dPieces->biright.arrangement[2].x = dPieces->biright.arrangement[3].x = START_LX+1;

	//Do Square
	dPieces->square.arrangement[0].y = dPieces->square.arrangement[1].y = START_LY;
	dPieces->square.arrangement[2].y = dPieces->square.arrangement[3].y = START_LY -1;
	dPieces->square.arrangement[0].x = dPieces->square.arrangement[2].x = START_LX -1;
	dPieces->square.arrangement[1].x = dPieces->square.arrangement[3].x = START_LX;
	
	return dPieces;
}


void drawGameArea()
{
	yzPenColor(.75* 65535,.75 * 65535, .75 * 65535);
	yzRect(0,0, WINDOW_WID, WINDOW_HGT);
	yzPenColor(65535*.5,65535*.5,65535*.5);
	yzRect(SIDE_BORDER-1, VERT_BORDER-1, SIDE_BORDER+BRICK_AREA_WID, WINDOW_HGT-VERT_BORDER-1);
	yzPenColor(0,0,0);
	yzRect(SIDE_BORDER, VERT_BORDER, SIDE_BORDER+BRICK_AREA_WID-1, WINDOW_HGT-VERT_BORDER-2);
}


void mainLoop()
{
	//level_t dLevel;
	int i;
	pieces_t *allPieces;
	tetris_colors_t *allColors;
	game_t *dGame;
	
	allColors = initColors();
	allPieces = initPieces(allColors);
	dGame = getGameDataFrom(GAME_FILE);
	
	
	if(!dGame)
		return;
		
	for(i=0; i < dGame->numLevels; i++)
	{	
		if(!handleActiveLevel(&(dGame->levels[i]), allPieces, dGame->defeatImage))
			break;
	}
	
	freeUpGame(dGame);
}


//Returns on true on success of level, false on failure
bool handleActiveLevel(level_t *aLevel, pieces_t *allPieces, yzImage *deathImage)
{
	int clocked, clockdelay;
	block_t activeBlock, oldBlock;
	event_t incomingEvent;
	bool go_flag;
	
	drawGameArea();
	
	if(aLevel->sideBarImage)
		drawSideImage(*(aLevel->sideBarImage));
	
	drawAll(aLevel);

	createRandomBlock(&activeBlock, allPieces);
	activeBlock.speed = aLevel->speed;
	clockdelay = CLOCKS_PER_SEC / aLevel->speed;
		
	clocked = clock();
	
	while(1)
	{
		//Handle Keyboard Move commands
		if(yzAreEventsWaiting())
		{
			incomingEvent = yzGetEvent(1);
			
			if(incomingEvent.type == YZ_KEY_DOWN)
			{
				go_flag = true;
				
				oldBlock = activeBlock;
				
				switch(incomingEvent.key)
				{
					case '4': case YZ_KEYCODE_LEFT_ARROW: 	moveBlockLeft(&activeBlock);		break;
					case '6': case YZ_KEYCODE_RIGHT_ARROW:	moveBlockRight(&activeBlock);		break;
					case '2': case YZ_KEYCODE_DOWN_ARROW:
							  activeBlock.speed = activeBlock.speed == aLevel->speed ? FAST_DROP_SPEED : aLevel->speed;
							  clockdelay = CLOCKS_PER_SEC / activeBlock.speed;
							  break;
					case '5': case '8': case YZ_KEYCODE_UP_ARROW: 	rotateBlock(&activeBlock); 	break;
					case 'q': return false;
					default : go_flag = false; break;
				}
				
				if(blockCollided(&activeBlock, aLevel))
					activeBlock =  oldBlock;
					
				else if(go_flag)
				{
					refreshBlock(&oldBlock);
					drawBlock(&activeBlock);
				}
			}
		}
		
		
		if(clock() - clocked >= clockdelay)
		{
			//Check to see if block has hit bottom
			if(hasHitBottom(&activeBlock, aLevel))
			{
				blockToBricks(&activeBlock, aLevel);
				checkForRows(aLevel);
				
				if(levelComplete(aLevel))
					return true;
				
				if(playerDied(aLevel))
				{
					if(deathImage)
						doFinalImage(*deathImage);
					
					return false;
				}
				
				createRandomBlock(&activeBlock, allPieces);
				activeBlock.speed = aLevel->speed;
				clockdelay =CLOCKS_PER_SEC / activeBlock.speed;
			}
			
			//Else Move Block Down
			else
			{
				refreshBlock(&activeBlock);
				moveBlockDown(&activeBlock);
				drawBlock(&activeBlock);
			}
			
			clocked =clock();
		}
	}
}


void createRandomBlock(block_t *activeBlock, pieces_t *dPieces)
{
	int randNum;
	
	randNum = rand() % 7;
	
	switch(randNum)
	{
		case 0: *activeBlock = dPieces->longy;		break;
		case 1: *activeBlock = dPieces->trileft;	break;
		case 2: *activeBlock = dPieces->triright;	break;
		case 3: *activeBlock = dPieces->trimid;		break;
		case 4: *activeBlock = dPieces->bileft;		break;
		case 5: *activeBlock = dPieces->biright;	break;
		case 6: *activeBlock = dPieces->square;		break;
	}
}


void moveBlockLeft(block_t *dBlock)
{
	int i;
	
	//Check to see if can go further left, if not abort
	for(i=0; i < BRICKS_IN_BLOCK; i++)
		if(dBlock->arrangement[i].x <= 0)
			return;
	
	//Move left
	for(i=0; i < BRICKS_IN_BLOCK; i++)
		dBlock->arrangement[i].x --;
}


void moveBlockRight(block_t *dBlock)
{
	int i;
	
	//Check to see if can go further right, if not abort
	for(i=0; i < BRICKS_IN_BLOCK; i++)
		if(dBlock->arrangement[i].x >= BRICKS_WIDE -1)
			return;
	
	//Move Right
	for(i=0; i < BRICKS_IN_BLOCK; i++)
		dBlock->arrangement[i].x ++;
}


void moveBlockDown(block_t *dBlock)
{
	int i;
	
	for(i=0; i < BRICKS_IN_BLOCK; i++)
		dBlock->arrangement[i].y --;
}


void rotateBlock(block_t *dBlock)
{
	int ulx = WINDOW_WID, uly = WINDOW_HGT, lrx = -1, lry=-1;
	int width, temp, i;
	
	for(i=0; i < BRICKS_IN_BLOCK; i++)
	{
		if(dBlock->arrangement[i].x  < ulx)
			ulx = dBlock->arrangement[i].x;
		if(dBlock->arrangement[i].x > lrx)
			lrx = dBlock->arrangement[i].x;
		if(dBlock->arrangement[i].y < uly)
			uly = dBlock->arrangement[i].y;
		if(dBlock->arrangement[i].y > lry)
			lry = dBlock->arrangement[i].y;
	}
	
	width = lrx - ulx;
	
	for(i=0; i < BRICKS_IN_BLOCK; i++)
	{
		dBlock->arrangement[i].x -= ulx;
		dBlock->arrangement[i].y -= uly;
		
		temp = dBlock->arrangement[i].y;
		dBlock->arrangement[i].y = dBlock->arrangement[i].x;
		dBlock->arrangement[i].x = temp;
				
		dBlock->arrangement[i].y= width - dBlock->arrangement[i].y; //may need to be changed, see notes
		
		dBlock->arrangement[i].x += ulx;
		dBlock->arrangement[i].y += uly;
	}
	
	//Prevent Rotating off screen
	for(i=0; i < BRICKS_IN_BLOCK; i++)
	{
		if(dBlock->arrangement[i].x >= BRICKS_WIDE -1)
		{
			moveBlockLeft(dBlock);
			i=0;
		}
		if(dBlock->arrangement[i].x <= 0)
		{
			moveBlockRight(dBlock);
			i=0;
		}
	}
}


//Checks to see if any rows are complete, if so it removes them
void checkForRows(level_t *dLevel)
{
	int i;
	bool redraw = false;
	
	for(i=0; i < BRICKS_HIGH; i++)
	{
		if(rowComplete(&(dLevel->rows[i])))
		{
			removeRow(dLevel, i);
			redraw = true;
			i=-1;
		}
	}
	
	if(redraw)
		drawAll(dLevel);
}


bool rowComplete(row_t *dRow)
{
	int i;
	
	for(i=0; i < BRICKS_WIDE; i++)
		if(!(dRow->row[i]))
			return false;
	
	return true;
}


//Level Num starts with 0 (bottom) and goes to BRICKS_HIGH - 1
void removeRow(level_t *aLevel, int level_num)
{
	int i;
	
	for(i=level_num; i < BRICKS_HIGH ; i++)
		aLevel->rows[i] = aLevel->rows[i+1];
}


bool levelComplete(level_t *aLevel)
{
	int i;
	
	for(i=0; i < BRICKS_HIGH; i++)
	{
		if(aLevel->rows[i].mustComplete == true)
			return false;
	}
	
	return true;
}


//Checks to see if any pieces are in top row
bool playerDied(level_t *aLevel)
{
	int i;
	
	for(i=0; i < BRICKS_WIDE; i++)
		if(aLevel->rows[START_LY].row[i])
			return true;
	return false;
}


void drawBrickAt(color_t *dColor, int x, int y)
{
	int sx, sy, ex, ey;
	
	if(dColor)
	{
		sx = SIDE_BORDER + x*BRICK_SIDE;
		sy = VERT_BORDER + BRICK_AREA_HGT - (y+1)*BRICK_SIDE;
		ex = SIDE_BORDER + (x +1)*BRICK_SIDE-1;	
		ey = VERT_BORDER + BRICK_AREA_HGT - y*BRICK_SIDE-1;
	
	
		if(sy < VERT_BORDER || ey > VERT_BORDER+BRICK_AREA_HGT || sx < SIDE_BORDER || ex > SIDE_BORDER + BRICK_AREA_WID)
			return;
		
		yzPenColor(65535,65535,65535);	
		yzRect(sx, sy, ex, ey);
		yzPenColor1c(*dColor);
		yzRect(sx +1, sy+1, ex -1, ey-1);
	}
}


void drawBrick(brick_t *dBrick)
{
	if(dBrick)
		drawBrickAt(dBrick->dColor, dBrick->x, dBrick->y);
}


//x & y are in blocks, not pixels
void drawBlankAt(int x, int y)
{
	int sx, sy, ex, ey;
	
	sx = SIDE_BORDER + x*BRICK_SIDE;
	sy = VERT_BORDER + BRICK_AREA_HGT - (y+1)*BRICK_SIDE;
	ex = SIDE_BORDER + (x +1)*BRICK_SIDE-1;	
	ey = VERT_BORDER + BRICK_AREA_HGT - (y)*BRICK_SIDE-1;
	
	
	if(sy < VERT_BORDER || ey > VERT_BORDER+BRICK_AREA_HGT || sx < SIDE_BORDER || ex > SIDE_BORDER + BRICK_AREA_WID)
		return;
		
	yzPenColor(0,0,0);	
	yzRect(sx, sy, ex, ey);
}


void drawBlock(block_t *dBlock)
{
	int i;
	
	for(i=0; i < BRICKS_IN_BLOCK; i++)
	{
		drawBrick(&(dBlock->arrangement[i]));
	}
}


void drawAll(level_t *aLevel)
{
	int i, j;
	
	for(i=0; i < BRICKS_HIGH; i++)
	{
		for(j=0; j < BRICKS_WIDE; j++)
		{
			if(!(aLevel->rows[i].row[j]))
				drawBlankAt(j, i);
			else
				drawBrickAt(aLevel->rows[i].row[j], j, i);
		}
	}
}


void refreshBrickAt(int x, int y)
{
	int sx, sy, ex, ey;
	
	sx = SIDE_BORDER + (x)*BRICK_SIDE;
	sy = VERT_BORDER + BRICK_AREA_HGT - (y+1)*BRICK_SIDE;
	ex = SIDE_BORDER + (x +1)*BRICK_SIDE-1;
	ey = VERT_BORDER + BRICK_AREA_HGT - (y)*BRICK_SIDE-1;


	if(sy < VERT_BORDER || ey > VERT_BORDER+BRICK_AREA_HGT || sx < SIDE_BORDER || ex > SIDE_BORDER + BRICK_AREA_WID)
		return;
	
	yzPenColor(0,0,0);
	yzRect(sx, sy, ex, ey);
}


void refreshBrick(brick_t *dBrick)
{
	if(dBrick)
		refreshBrickAt(dBrick->x, dBrick->y);
}


void refreshBlock(block_t *dBlock)
{
	int i;
	
	for(i=0; i < BRICKS_IN_BLOCK; i++)
		refreshBrick(&(dBlock->arrangement[i]));
}


void drawSideImage(yzImage aImage)
{	
	//Check to make sure conforms to x & y Maxes
	if(getImageWidth(aImage) > MAX_IMAGE_WID || getImageHeight(aImage) > MAX_IMAGE_HGT)
		return;
		
	//Draw Image (calc x & y for centering)
	drawImage(aImage, IMAGE_CX - getImageWidth(aImage)/2, IMAGE_CY - getImageHeight(aImage)/2);
}


void blockToBricks(block_t *dBlock, level_t *allBricks)
{
	int i;
	
	for(i=0; i < BRICKS_IN_BLOCK; i++)
		allBricks->rows[dBlock->arrangement[i].y].row[dBlock->arrangement[i].x] = dBlock->arrangement[i].dColor;
}


bool blockCollided(block_t *dBlock, level_t *allBricks)
{
	 int i;
	 
	 for(i=0; i < BRICKS_IN_BLOCK; i++)
	 	if(allBricks->rows[dBlock->arrangement[i].y].row[dBlock->arrangement[i].x])
			return true;
	 
	 return false;
}


bool hasHitBottom(block_t *dBlock, level_t *allBricks)
{
	int i;
		
	for(i=0; i < BRICKS_IN_BLOCK; i++)
	{
		if(dBlock->arrangement[i].y == 0)
			return true;
			
		else if(allBricks->rows[dBlock->arrangement[i].y -1].row[dBlock->arrangement[i].x])
			return true;
	}
	
	return false;
}


void freeUpGame(game_t *dGame)
{
	int i;
	
	for (i=0; i < dGame->numLevels ; i++)
	{
		if(dGame->levels[i].sideBarImage)
		{
			destroyImage(*(dGame->levels[i].sideBarImage));
			free(dGame->levels[i].sideBarImage);
		}
	}
			
	
	free(dGame->levels);
	
	if(dGame->defeatImage)
		free(dGame->defeatImage);
	
	if(dGame->victoryImage)
		free(dGame->victoryImage);
		
	free(dGame->allColors->blue);
	free(dGame->allColors->red);
	free(dGame->allColors->green);
	free(dGame->allColors->orange);
	free(dGame->allColors->purple);
	free(dGame->allColors->yellow);
	free(dGame->allColors->brown);
	free(dGame->allColors);
	free(dGame);
}


void doFinalImage(yzImage dImage)
{
	event_t incomingEvent;
	int sy, sx;
	
	sx = WINDOW_CX - getImageWidth(dImage)/2;
	sy = WINDOW_CY - getImageHeight(dImage)/2;
	
	drawImage(dImage, sx, sy);
	
	while(1)
	{
		if(yzAreEventsWaiting())
		{
			incomingEvent = yzGetEvent(1);
			
			if(incomingEvent.type == YZ_KEY_DOWN)
				return;
		}
	}
}