/*********************************
 * Steven Stanek
 * June 13, 2001
 * tetrisfilereader.c
 *********************************/
 
#include "tetrisfilereader.h"
#include <string.h>
#undef 	 feof

#define INTRO_STR 		"<BEGIN YZTetris FILE VERSION "
#define END_FILE		"<END FILE>"
#define PRES_VERSION	"0.0.0a"
#define FN_MAX_LN 		64
#define	LEVEL_INTRO		"<BEGIN LEVEL SPEED="
#define END_LEVEL		"<END LEVEL>"
#define START_BLOCKS	"<BEGIN BLOCKS>"
#define END_BLOCKS		"<END BLOCKS>"
#define BEGIN_IMAGE		"<BEGIN IMAGE>"
#define END_IMAGE		"<END IMAGE>"
#define BEGIN_VICTORY	"<BEGIN VICTORY IMAGE>"
#define END_VICTORY		"<END VICTORY IMAGE>"
#define BEGIN_DEFEAT	"<BEGIN DEFEAT IMAGE>"
#define END_DEFEAT		"<END DEFEAT IMAGE>"


static level_t *readLevel(FILE *infile, tetris_colors_t *allColors, int level); //Returns null if no more levels are left
static level_t *processFirstLevelStr(FILE* infile, game_t *dGame); //Returns true to keep, reading false to stop
static int getSpeedFromLevelStart(char *lstart);
static row_t* readRow(FILE* infile, tetris_colors_t *allColors); //Returns null if at end
static void assignYCoords(level_t *dLevel, int numRows);
static color_t* charToColor(char toConvert, tetris_colors_t *allColors);
static void readEndGameImages(FILE *infile, game_t *dGame);

game_t *askForGameData()
{	
	char fn[FN_MAX_LN];
	FILE *infile;
	game_t *dGame;
	
	while(1)
	{
		printf("Please enter the filename of the tetris levels file\n");
		scanf("%s", fn);
		infile = fopen(fn, "r");
		
		if(infile)
			break;
		
		printf("There was a problem reading the file, please reenter the filename\n");
	}
	
	dGame=getGameData(infile);
	fclose(infile);
	return dGame;
}


game_t *getGameDataFrom(char *fn)
{
	FILE *infile;
	game_t *dGame;
	
	infile=fopen(fn,"r");
	dGame=getGameData(infile);
	fclose(infile);
	return dGame;
}


game_t *getGameData(FILE *infile)
{
	game_t* dGame;
	int i;
	char in_buffer[128];
	level_t *levels[MAX_NUM_LEVELS], *presLevel;
	
	assert(dGame = malloc(sizeof(game_t)));
	
	
	dGame->defeatImage = dGame->victoryImage = 0;
	
	dGame->allColors = initColors();
	
	//Check Intro
	fgets(in_buffer, strlen(INTRO_STR)+1, infile);
	
	if(strcmp(in_buffer, INTRO_STR))
	{
		printf("File formatting error in lead in content\n");
		return 0;
	}
	
	//Check Version
	fgets(in_buffer, strlen(PRES_VERSION)+1, infile);
	
	if(strcmp(in_buffer, PRES_VERSION))
	{
		printf("File formatting error in Version #\n");
		return 0;
	}
		
	//Read in Line Break
	fgets(in_buffer, 128,  infile);
	
	//Read in Levels & set numlevels
	presLevel = processFirstLevelStr(infile, dGame);
	
	for(i=0; presLevel && i < MAX_NUM_LEVELS; i++)
	{
		levels[i] = presLevel;
		presLevel = processFirstLevelStr(infile, dGame);
	}
	
	dGame->numLevels = i;
	
	assert(dGame->levels = malloc(dGame->numLevels*sizeof(level_t)));
	for(i=0; i < dGame->numLevels; i++)
	{
		dGame->levels[i] = *levels[i];
		free(levels[i]);
	}
	
	return dGame;
}


static level_t *readLevel(FILE *infile, tetris_colors_t *allColors, int level)
{
	level_t *aLevel;
	char in_buffer[128];
	row_t *presRow;
	int i, j;
	
	assert(aLevel = malloc(sizeof(level_t)));
	
	//Set Must Complete Fields, Make all squares empty
	for(i=0; i <= BRICKS_HIGH; i++)
	{
		aLevel->rows[i].mustComplete = false;
		
		for(j=0; j < BRICKS_WIDE; j++)
		{
			aLevel->rows[i].row[j] = 0;
		}
	}
	
	//Read Speed
	aLevel->speed = level;
	
	if(aLevel->speed > TOP_LEVEL_SPEED || aLevel->speed < LOW_LEVEL_SPEED)
	{
		printf("File formatting error in speed\n");
		return 0;
	}
	
	//Read Begin Blocks
	fgets(in_buffer, strlen(START_BLOCKS)+1, infile);
	
	if(strcmp(in_buffer, START_BLOCKS))
	{
		printf("File formatting error in beginning of blocks tag content\n");
		return 0;
	}
	
	//Read Line Break
	fgets(in_buffer, 128, infile);
	
	//Read Rows
	presRow=readRow(infile, allColors);
	for(i=0; presRow; i++)
	{
		aLevel->rows[i] = *presRow;
		free(presRow);
		presRow=readRow(infile, allColors);
	}
	
	if(i < MIN_NUM_ROWS || i > MAX_NUM_ROWS)
	{
		printf("Too many or two few rows in level");
		exit(1);
	}
	
	//Assign y Coords
	assignYCoords(aLevel, i);
	
	//Read End Blocks
	aLevel->sideBarImage = 0;
	
	fgets(in_buffer, strlen(END_BLOCKS)+1, infile);
	
	if(strcmp(in_buffer, END_BLOCKS))
	{
		printf("End of Blocks didn't work right\n");
		return 0;
	}
	
	fgets(in_buffer, 128, infile); // Get rid of trailing \n
	
	//Read End Level or Image
	fgets(in_buffer, 128, infile);
	if(strncmp(in_buffer, END_LEVEL, strlen(END_LEVEL)) == 0)
		return aLevel;
	
	//Read an decide on last tag
	else if(strncmp(in_buffer, BEGIN_IMAGE, strlen(BEGIN_IMAGE)))
	{
		printf("End String is wrong\n");
		return 0;
	}
	else
	{	
		//Read Tailing Tag (see if image)
		assert(aLevel->sideBarImage = malloc(sizeof(yzImage)));
		*(aLevel->sideBarImage) = openImageFile(infile);
		fgets(in_buffer, 128, infile);
		
		if(strncmp(in_buffer, END_IMAGE, strlen(END_IMAGE)))
		{
			printf("End Image String is wrong\n");
			return 0;
		}
		
		//If image- read end of level tag
		fgets(in_buffer, 128, infile);
		
		if(strncmp(in_buffer, END_LEVEL, strlen(END_LEVEL)))
		{
			printf("End String is wrong\n");
			return 0;
		}
		
		return aLevel;
	}
}


/* Function: static level_t *processFirstLevelStr(FILE* infile, game_t *dGame)
 * This function reads the first line of a level (or any line preceding a level
 * and determines what to do with it. Since the base case of this function is
 * reading the next level, most of the time it returns a level_t * for the next
 * level. However, if the file is finished 
 */
static level_t *processFirstLevelStr(FILE* infile, game_t *dGame)
{
	char in_buffer[128];
	
	fgets(in_buffer, 128, infile);
	
	if(feof(infile) || !strncmp(in_buffer, END_FILE, strlen(END_FILE)))
		return 0;
	else if(!strncmp(in_buffer, LEVEL_INTRO, strlen(LEVEL_INTRO)))
		return readLevel(infile, dGame->allColors, getSpeedFromLevelStart(in_buffer));
	else if(!strncmp(in_buffer, BEGIN_VICTORY, strlen(BEGIN_VICTORY)))
	{
		readEndGameImages(infile, dGame);
		return 0;
	}
	else
	{
		printf("The string \"%s\" is misplaced or badly formatted", in_buffer);
		return 0;
	}
}


static int getSpeedFromLevelStart(char *lstart)
{
	int toreturn;
	
	lstart += strlen(LEVEL_INTRO);
	sscanf(lstart, "%d", &toreturn);
	
	if(toreturn > MAX_NUM_LEVELS || toreturn  < MIN_NUM_LEVELS)
	{
		printf("NUM Levels is wrong");
		exit(2);
	}
	
	return toreturn;
}


static row_t* readRow(FILE* infile, tetris_colors_t *allColors)
{
	char firstchar, row_buf[BRICKS_WIDE];
	row_t* aRow;
	int i;

	firstchar = (char) fgetc(infile);
	ungetc((int)firstchar, infile);
	
	if(firstchar == '<')
		return 0;
	
	fgets(row_buf, BRICKS_WIDE+1, infile);
	
	assert(aRow =  malloc(sizeof(row_t)));
	aRow->mustComplete = true;
	
	for(i=0; i < BRICKS_WIDE; i++)
		aRow->row[i] = charToColor(row_buf[i], allColors);
	
	//Get rid of Line break
	fgets(row_buf, BRICKS_WIDE, infile);
	
	if(row_buf[0] != '\n')
	{
		printf("Row was not terminated!\n");
		return 0;
	}
	
	return aRow;
}


static void assignYCoords(level_t *dLevel, int numRows)
{
	int i;
	row_t tempRow;
	
	for(i=0; i < (numRows+1)/2; i++)
	{
		//Swap i, numRows-i
		tempRow = dLevel->rows[i];
		dLevel->rows[i] = dLevel->rows[numRows-i-1];
		dLevel->rows[numRows-i-1] = tempRow;
	}
}


static color_t* charToColor(char toConvert, tetris_colors_t *allColors)
{
	switch (toConvert)
	{
		case '+': return 0;
		case 'r': return allColors->red; 	
		case 'b': return allColors->blue; 	
		case 'p': return allColors->purple;	
		case 'o': return allColors->orange;	
		case 'y': return allColors->yellow;	
		case 'w': return allColors->brown;
		case 'g': return allColors->green;
		default: printf("Invalid Character in file.\n"); return 0;
	}
}


static void readEndGameImages(FILE *infile, game_t *dGame)
{
	char in_buffer[128];
	
	//Read in Victory Image
	assert(dGame->victoryImage = malloc(sizeof(yzImage)));
	*(dGame->victoryImage) = openImageFile(infile);
	
	//Read in End Tag 
	fgets(in_buffer, 128, infile);
	
	//Verify End Tag
	if(strncmp(END_VICTORY, in_buffer, strlen(END_VICTORY)))
	{
		printf("Error in reading termination of victory image");
		exit(3);
	}
	
	//Read Defeat Start Tag
	fgets(in_buffer, 128, infile);
	
	//Verify Defeat Start Tag
	if(strncmp(BEGIN_DEFEAT, in_buffer, strlen(BEGIN_DEFEAT)))
	{
		printf("Error in reading start of defeat image");
		exit(4);
	}
	
	//Read in Defeat Image
	assert(dGame->defeatImage = malloc(sizeof(yzImage)));
	*(dGame->defeatImage) = openImageFile(infile);

	//Read in Defeat Image End Tag
	fgets(in_buffer, 128, infile);
	
	//Very end of defeat tag
	if(strncmp(END_DEFEAT, in_buffer, strlen(END_DEFEAT)))
	{
		printf("Error in reading start of defeat image");
		exit(5);
	}
}